
import logging
logger = logging.getLogger(__name__)



from numpy import *
import numpy as np
import os
import functools
import math

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from time import sleep, time

from collections import OrderedDict 


from spimagine.models.keyframe_model import KeyFrame, KeyFrameList
from spimagine.models.data_model import DataModel, DemoData


from spimagine.gui.gui_utils import  createImageCheckbox,createStandardButton

from spimagine.models.transform_model import TransformModel


import spimagine




def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    import sys

    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        logger.debug("found MEIPASS: %s "%os.path.join(base_path, os.path.basename(myPath)))

        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:

        base_path = os.path.abspath(os.path.dirname(__file__))
        logger.debug("didnt found MEIPASS!: %s "%os.path.join(base_path, myPath))

        return os.path.join(base_path, myPath)



class KeyEdge(QGraphicsItem):
    def __init__(self, sourceKeyNode, destKeyNode):
        super(KeyEdge, self).__init__()

        self.sourcePoint = QPointF()
        self.destPoint = QPointF()

        # self.setAcceptedMouseButtons(Qt.NoButton)
        # self.setFlag(QGraphicsItem.ItemIsMovable)
        # self.setFlag(QGraphicsItem.ItemIsSelectable)
        self.setFlag(QGraphicsItem.ItemSendsGeometryChanges)
        # self.setFlag(QGraphicsItem.ItemIgnoresTransformations)
        self.setCacheMode(QGraphicsItem.DeviceCoordinateCache)
        self.setZValue(0)


        self.source = sourceKeyNode
        self.dest = destKeyNode
        self.source.addKeyEdge(self)
        self.dest.addKeyEdge(self)
        self.adjust()


    def itemChange(self, change, value):
        if change == QGraphicsItem.ItemPositionHasChanged:
            logger.debug("changed")
        if change == QGraphicsItem.ItemSelectedChange:
            logger.debug("selected")

        return super(KeyEdge, self).itemChange(change, value)


    def adjust(self):
        if not self.source or not self.dest:
            return

        line = QLineF(self.mapFromItem(self.source, 0, 0),
                self.mapFromItem(self.dest, 0, 0))
        length = line.length()

        self.prepareGeometryChange()
        self.sourcePoint = line.p1()
        self.destPoint = line.p2()

    def boundingRect(self):
        if not self.source or not self.dest:
            return QRectF()

        penWidth = 1.0
        extra = 5

        return QRectF(self.sourcePoint,
                QSizeF(self.destPoint.x() - self.sourcePoint.x(),
                        self.destPoint.y() - self.sourcePoint.y()+10)).normalized().adjusted(-extra, -extra, extra, extra)

    def paint(self, painter, option, widget):
        if not self.source or not self.dest:
            return

        # Draw the line itself.
        line = QLineF(self.sourcePoint, self.destPoint)

        if line.length() == 0.0:
            return

        if option.state & QStyle.State_Sunken:
            painter.setPen(QPen(Qt.lightGray, 2, Qt.SolidLine))
        else:
            painter.setPen(QPen(Qt.gray, 2, Qt.SolidLine))

        painter.drawLine(line)


class KeyNode(QGraphicsItem):
    def __init__(self,graphWidget, transform,keyList, ID,fixed = False):
        super(KeyNode, self).__init__()
        self.graph = graphWidget

        self.shapeSize = 8*array([-1,-1.,2,2])
        self.edgeList = []
        self.keyList = keyList
        self.transformModel = transform
        self.ID = ID
        pos = self.keyList[self.ID].pos*KeyFrameScene.WIDTH

        self.setPos(QPointF(pos,0))

        self.fixed = fixed

        if not self.fixed:
            self.setFlag(QGraphicsItem.ItemIsMovable)

        self.setFlag(QGraphicsItem.ItemIsSelectable)
        self.setFlag(QGraphicsItem.ItemSendsGeometryChanges)
        self.setFlag(QGraphicsItem.ItemIgnoresTransformations)
        # self.setCacheMode(QGraphicsItem.DeviceCoordinateCache)
        if fixed:
            self.setZValue(0.5)
        else:
            self.setZValue(1)
        self.setToolTip("Keynode")
        # self.updateTransformData()

    def addKeyEdge(self, edge):
        self.edgeList.append(edge)
        edge.adjust()

    def edges(self):
        return self.edgeList


    def boundingRect(self):
        return QRectF(*self.shapeSize)

    def shape(self):
        path = QPainterPath()
        path.addEllipse(*self.shapeSize)
        return path

    def paint(self, painter, option, widget):
        gradient = QRadialGradient(.2*self.shapeSize[0],
                                         .2*self.shapeSize[1], .3*self.shapeSize[2])
        if option.state & (QStyle.State_Sunken | QStyle.State_Selected) :
            gradient.setColorAt(1, QColor(51, 153, 204,255))
            gradient.setColorAt(0, QColor(255,255,255,255))

        else:
            gradient.setColorAt(0, Qt.gray)
            gradient.setColorAt(1, Qt.darkGray)

        painter.setBrush(QBrush(gradient))

        painter.setPen(QPen(Qt.black, 0))
        painter.setPen(QPen(Qt.transparent, 0))

        # painter.drawEllipse(*self.shapeSize)
        rect = QRect(*self.shapeSize)
        # painter.drawRect(rect)
        painter.drawRoundedRect(rect,70.,70.,mode=Qt.RelativeSize)

        # painter.drawPie(*(list(self.shapeSize)+[0,180*16]))

    def itemChange(self, change, value):
        if change == QGraphicsItem.ItemPositionHasChanged:
            sceneRect = self.graph.scene.sceneRect()
            x = value.toPoint().x()
            x1, x2 = sceneRect.x(),sceneRect.x()+sceneRect.width()
            pos = QPointF(clip(x,x1,x2),0)
            tpos = clip(1.*pos.x()/KeyFrameScene.WIDTH,0.,1.)
            # self.keyList[self.ID].tFrame = tFrame

            self.keyList.update_pos(self.ID, tpos)
            self.setPos(pos)
            for edge in self.edgeList:
                edge.adjust()
            self.setToolTip("KeyNode: t= %.2f"%self.pos().x())

            self.graph.itemMoved()
            # print self.graph.keyList


        elif change == QGraphicsItem.ItemSelectedChange:
            logger.debug("keynode selected")

        return super(KeyNode, self).itemChange(change, value)

    def mousePressEvent(self, event):
        self.update()
        super(KeyNode, self).mousePressEvent(event)

    def mouseReleaseEvent(self, event):
        self.update()
        super(KeyNode, self).mouseReleaseEvent(event)

    def mouseDoubleClickEvent(self,event):
        super(KeyNode,self).mouseDoubleClickEvent(event)
        self.setTransformData()

    def delete(self):
        self.keyList.removeItem(self.ID)
        # self.graph.resetScene()


    def showProperties(self):
        QMessageBox.about(None, "KeyFrame", str(self.keyList[self.ID]))


    def updateTransformData(self):
        if self.transformModel is None:
            self.keyList[self.ID].transformData = TransformData()
        else:
            self.keyList[self.ID].transformData = self.transformModel.toTransformData()

    def setTransformData(self):
        self.transformModel.fromTransformData(self.keyList[self.ID].transformData)

    def contextMenuEvent(self, contextEvent):
        actionMethods = OrderedDict((("update",self.updateTransformData),
                                     ("delete",self.delete),
                                     ("properties" ,self.showProperties)))

        actions = OrderedDict()

        object_cntext_Menu = QMenu()
        for k, meth in actionMethods.iteritems():
            actions[k] = object_cntext_Menu.addAction(k,meth)

        if self.fixed:
            actions["delete"].setEnabled(False)

        position=QCursor.pos()
        object_cntext_Menu.exec_(position)


class KeyFrameScene(QGraphicsScene):
    WIDTH = 100
    HEIGHT = 10

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.accept()
        else:
            event.ignore()

    def dragMoveEvent(self, event):
        event.accept()


    # def __init__(self, parent):
    #     super(KeyFrameScene, self).__init__(parent)



class KeyListView(QGraphicsView):
    def __init__(self):
        super(KeyListView, self).__init__()

        self.setVerticalScrollBarPolicy (Qt.ScrollBarAlwaysOff)
        self.setCacheMode(QGraphicsView.CacheBackground)
        self.setViewportUpdateMode(QGraphicsView.BoundingRectViewportUpdate)
        self.setRenderHint(QPainter.Antialiasing)
        self.setTransformationAnchor(QGraphicsView.AnchorUnderMouse)
        self.setResizeAnchor(QGraphicsView.AnchorViewCenter)
        self.setSizePolicy(QSizePolicy.Minimum,QSizePolicy.Fixed)

        self.scene = KeyFrameScene(self)

        self.setMinimumHeight(30)
        self.setMaximumHeight(30)

        self.scene.setItemIndexMethod(QGraphicsScene.NoIndex)
        self.scene.setSceneRect(0, -KeyFrameScene.HEIGHT/2, KeyFrameScene.WIDTH, KeyFrameScene.HEIGHT)

        self.setScene(self.scene)

        # self.scale(1,.1)

        # self.setMinimumSize(300,20)
        self.setWindowTitle("KeyFrameView")
        self.zoom = 1.
        self.relativeAspect = 1.
        self.isListening = True

        self.connect_to_transform(None)
        self.setModel(KeyFrameList())

    def connect_to_transform(self,transformModel):
        self.transformModel = transformModel

    # def reset(self,transformModel):
    #     self.keyList = KeyFrameList()

    #     self.keyList.addItem(KeyFrame(0.))
    #     self.keyList.addItem(KeyFrame(1.))

    #     print self.keyList

    #     logger.debug("reset, : keyList = %s"%self.keyList)
    #     self.transformModel = transformModel
    #     self.resetScene()
    #     self.keyList._modelChanged.connect(self.modelChanged)


    # def resetModels(self,transformModel,keyList= KeyFrameList):
    #     self.keyList = keyList
    #     logger.debug("resetModels, : keyList = %s"%self.keyList)
    #     self.transformModel = transformModel
    #     self.resetScene()
    #     self.keyList._modelChanged.connect(self.modelChanged)

    def setModel(self,keyList = KeyFrameList()):
        logger.debug("setModel: %s",keyList)
        self.keyList = keyList
        self.resetScene()
        # for it in self.scene.items():
        #     it.updateTransformData()

        self.keyList._modelChanged.connect(self.modelChanged)

    # def setKeyListModel(self,keyList):
    #     self.keyList = keyList
    #     self.resetScene()
    #     self.keyList._modelChanged.connect(self.modelChanged)
        # self.keyList._itemChanged.connect(self.itemChanged)


    # def setTransformModel(self,transformModel):
    #     self.transformModel = transformModel
    #     self.resetScene()
    #     # self.keyList._modelChanged.connect(self.modelChanged)
    #     # self.keyList._itemChanged.connect(self.itemChanged)

    def resetScene(self):
        logger.debug("resetScene: %s",self.keyList)
        self.scene.clear()
        for i, (pos,ID) in enumerate(self.keyList.posdict.iteritems()):
            fixed = ( i == 0 or i == len(self.keyList.posdict)-1)
            self.scene.addItem(KeyNode(self,self.transformModel,self.keyList,ID,fixed))


    def modelChanged(self):
        logger.debug("model Changed")
        self.resetScene()


    def addKey(self,keyFrame, fixed = False):
        pass

    def itemMoved(self):
        pass

    def keyPressEvent(self, event):
        key = event.key()

        super(KeyListView, self).keyPressEvent(event)

    def resizeEvent(self,event):
        self.relativeAspect = 1.*event.size().width()/KeyFrameScene.WIDTH
        self.setTransform(QTransform.fromScale(self.relativeAspect*self.zoom, 1.))


    def wheelEvent(self, event):
        factor = 1.41 ** (-event.delta() / 240.0)
        self.zoom = clip(self.zoom*factor,1.,1.e3)

        self.setTransform(QTransform.fromScale(self.relativeAspect*self.zoom, 1.))

    def drawBackground(self,painter, rect):
        sceneRect = self.sceneRect()
        # painter.setBrush(Qt.black)
        painter.setBrush(QColor(50,50,50))
        painter.drawRect(sceneRect)


    def contextMenuEvent(self, event):
        # super(KeyListView,self).contextMenuEvent(event)

        item = self.scene.itemAt(self.mapToScene(event.pos()))
        if type(item) == KeyNode:
            super(KeyListView,self).contextMenuEvent(event)
            return

        else:
            posScene = self.mapToScene(event.pos())

            actionMethods = {"insert keyframe" : functools.partial(self.keyList.addItem,KeyFrame(1.*posScene.x()/KeyFrameScene.WIDTH,self.transformModel.toTransformData()))}
            actions = {}

            object_cntext_Menu = QMenu()
            for k, meth in actionMethods.iteritems():
                actions[k] = object_cntext_Menu.addAction(k,meth)

                object_cntext_Menu.exec_(self.mapToGlobal(event.pos()))


    def load_from_JSON(self,fName):
        with open(fName,"r") as f:
            try:
                k = KeyFrameList._from_JSON(f.read())
                self.setModel(k)
                
            except Exception as e:
                print e
                print "not a valid keyframe json file: %s"%fName
                

    def dropEvent(self, event):
        logger.debug("dropping...")
        for url in event.mimeData().urls():
            event.accept()
            path = url.toLocalFile().toLocal8Bit().data()

            if spimagine.config.__SYSTEM_DARWIN__:
                path = spimagine.config._parseFileNameFix(path)

            self.load_from_JSON(path)



class KeyFramePanel(QWidget):
    _keyTimeChanged = pyqtSignal(float)


    def __init__(self, glWidget):
        super(KeyFramePanel,self).__init__()
        self.glWidget = glWidget
        self.resize(500, 30)
        self.initUI()


    def initUI(self):
        self.keyView =  KeyListView()

        self.setAcceptDrops(True)

        self.playTimer = QTimer(self)
        self.playTimer.setInterval(30)
        self.playTimer.timeout.connect(self.onPlayTimer)
        self.recordTimer = QTimer(self)
        self.recordTimer.setInterval(30)
        self.recordTimer.timeout.connect(self.onRecordTimer)


        self.playButton = createStandardButton(self,
                        fName = absPath("images/icon_start.png"),
                        method = self.onPlay,
                        tooltip = "play animation")
        self.playButton.setMaximumWidth(24)
        self.playButton.setMaximumHeight(24)


        self.recordButton = createStandardButton(self,
                        fName = absPath("images/icon_record.png"),
                        method = self.onRecord,
                        tooltip = "render images to folder")

        self.recordButton.setMaximumWidth(24)
        self.recordButton.setMaximumHeight(24)


        self.distributeButton  = createStandardButton(self,
                        fName = absPath("images/icon_distribute.png"),
                        method = self.onDistribute,
                        tooltip = "sync data timepoints to data keyframe position")

        self.distributeButton.setMaximumWidth(24)
        self.distributeButton.setMaximumHeight(24)

        self.saveButton = createStandardButton(self,
                        fName = absPath("images/icon_save.png"),
                        method = self.onSave,
                        tooltip = "save keyframes as json")


        self.saveButton.setMaximumWidth(24)
        self.saveButton.setMaximumHeight(24)

        self.trashButton =createStandardButton(self,
                        fName = absPath("images/icon_trash.png"),
                        method = self.onTrash,
                        tooltip = "delet all keyframes")

        self.trashButton.setMaximumWidth(24)
        self.trashButton.setMaximumHeight(24)


        self.slider = QSlider(Qt.Horizontal)
        self.slider.setTickPosition(QSlider.TicksBothSides)
        self.slider.setTickInterval(1)
        self.slider.setFocusPolicy(Qt.ClickFocus)
        self.slider.setTracking(False)

        self.slider.setRange(0,100)
        self.slider.setStyleSheet("height: 12px; border = 0px;")


        hbox = QHBoxLayout()


        hbox.addWidget(self.playButton)
        hbox.addWidget(self.recordButton)
        hbox.addWidget(self.distributeButton)
        hbox.addWidget(self.saveButton)

        hbox.addWidget(self.trashButton)
        hbox.addWidget(self.keyView)

        vbox = QVBoxLayout()
        vbox.addLayout(hbox)
        vbox.addWidget(self.slider)

        def _set_value_without_emitting(val):
            old = self.slider.blockSignals(True)
            self.slider.setValue(int(100*val))
            self.slider.blockSignals(old)
            
        self._keyTimeChanged.connect(_set_value_without_emitting)
        
        self.slider.valueChanged.connect(lambda x:self.setKeyTime(x/100.))

        self.setLayout(vbox)
        self.setFrameNumber(100)

        self.setDirName("./")
        self.t = 0


    def connect_to_transform(self, transform = None ):
        if not transform:
            transform = TransformModel()
            
        logger.debug("keyPanel.connect_to_transform\n")
        self.keyView.connect_to_transform(transform)

    def setModel(self,keyList = None):
        if not keyList:
            keyList=KeyFrameList()
            
        logger.debug("keyPanel.setModel: keyList = %s\n"%keyList)
        self.keyView.setModel(keyList)

    # def resetModels(self,transformModel,keyList=KeyFrameList()):
    #     logger.debug("keyPanel.resetModel: keyList = %s\n"%keyList)
    #     self.transformModel = transformModel
    #     self.keyView.resetModels(transformModel,keyList)

    # def reset(self,transformModel):
    #     self.transformModel = transformModel
    #     self.keyView.reset(transformModel)

    def onPlay(self,evt):
        if self.playTimer.isActive():
            self.playTimer.stop()
            self.playButton.setIcon(QIcon(absPath("images/icon_start.png")))

        else:
            self.playTimer.start()
            self.playButton.setIcon(QIcon(absPath("images/icon_pause.png")))


    def onDistribute(self,e):
        if self.keyView.transformModel:
            pos1 = self.keyView.keyList.getTransform(0).dataPos
            pos2 = self.keyView.keyList.getTransform(1.).dataPos
            self.keyView.keyList.distribute(pos1, pos2)
            # self.keyView.keyList.distribute()

            
    def onRecord(self,evt):
        if self.recordTimer.isActive():
            self.recordTimer.stop()
            self.recordButton.setIcon(QIcon(absPath("images/icon_record.png")))
        else:
            self.recordPos = 0
            self.recordButton.setIcon(QIcon(absPath("images/icon_record_on.png")))
            self.recordTimer.start()


    def setFrameNumber(self,nFrames):
        self.nFrames = nFrames

    def setDirName(self,dirName):
        logger.debug("setDirName %s"%dirName)
        self.dirName = str(dirName)

    def onRecordTimer(self):
        self.recordPos  += 1
        if self.recordPos > self.nFrames:
            self.recordTimer.stop()
            self.recordButton.setIcon(QIcon(absPath("images/icon_record.png")))
            return

        self.setKeyTime(1.*self.recordPos/self.nFrames)

        self.glWidget.saveFrame(os.path.join(self.dirName,"output_%s.png"%(str(self.recordPos).zfill(int(log10(self.nFrames)+1)))))



    def setKeyTime(self,newTime):
        self.t = np.clip(newTime,0,1.)
        logger.debug("set key time to %s"%self.t)

        if self.keyView.transformModel:
            self.keyView.transformModel.fromTransformData(self.keyView.keyList.getTransform(newTime))


        self._keyTimeChanged.emit(self.t)


    def onPlayTimer(self):
        self.setKeyTime((self.t+0.01)%1.)

    def onSave(self):

        fName = QFileDialog.getSaveFileName(self, "save as json file", "", "json files (*.json)")
        if fName:
            self.save_to_JSON(fName)


    def onTrash(self):
        self.keyView.setModel(KeyFrameList())

    def save_to_JSON(self,fName):
        with open(fName,"w") as f:
            f.write(self.keyView.keyList._to_JSON())

    def load_from_JSON(self,fName):
        self.keyView.load_from_JSON(fName)
            

class MainWindow(QMainWindow):

    def __init__(self):
        super(MainWindow,self).__init__()

        self.resize(500, 40)
        self.setWindowTitle("Key Frame View")



        self.keyPanel = KeyFramePanel(None)

        dataModel = DataModel(dataContainer = DemoData(50),prefetchSize = 0)
        transModel = TransformModel()
        transModel.setModel(dataModel)

        transModel.setValueScale(0,200)
        dataModel.setPos(2)
        self.keyPanel.connect_to_transform(transModel)


        k = KeyFrameList()
        k.addItem(KeyFrame(0.))
        k.addItem(KeyFrame(1.))

        self.keyPanel.setModel(k)


        self.setCentralWidget(self.keyPanel)

        self.setStyleSheet("background-color:black;")


    # def resizeEvent(self,event):
    #     newSize = event.size()
    #     newSize.setHeight(10)
    #     self.resize(newSize)



class FooWidget(QWidget):
    def __init__(self):
        super(FooWidget,self).__init__()

        foo = KeyListView()


        slider = QSlider(Qt.Horizontal)
        slider.setTickPosition(QSlider.TicksBothSides)
        slider.setTickInterval(1)
        slider.setFocusPolicy(Qt.ClickFocus)
        slider.setTracking(True)

        vbox = QVBoxLayout()
        vbox.addWidget(foo)
        vbox.addWidget(slider)

        self.setLayout(vbox)

class MainWindowEmpty(QMainWindow):

    def __init__(self):
        super(MainWindowEmpty,self).__init__()

        self.setWindowTitle("Key Frame View")

        self.foo = FooWidget()



        self.setCentralWidget(self.foo)
        self.resize(500,60)
        self.setStyleSheet("background-color:black;")

if __name__ == '__main__':

    import sys

    app = QApplication(sys.argv)

    win = MainWindow()

    # win = MainWindowEmpty()

    win.show()
    win.raise_()

    app.exec_()



import sys

import numpy as np

from PyQt4 import QtCore, QtGui, Qt
from sortedcontainers import SortedDict

from spimagine.models.imageprocessor import ImageProcessor

from spimagine.gui import gui_utils

class ImageFlow(QtCore.QObject):
    # _dataPosChanged = QtCore.pyqtSignal(int)

    def __init__(self):
        self.processors = SortedDict()


    def add_processor(self,processor):
        self.processors[len(self.processors)] = processor

    def apply(self,data):
        if len(self.processors) == 0:
            return data

        data = data.copy()
        for p in self.processors.values():
            data = p.apply(data)

        return data




def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    import os
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:
        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)


checkBoxStyleStr = """
    QCheckBox::indicator:checked {
    background:black;
    border-image: url(%s);}
    QCheckBox::indicator:unchecked {
    background:black;
    border-image: url(%s);}
    """

def createImgCheckBox(fName_active,fName_inactive):
    checkBox = QtGui.QCheckBox()
    checkBox.setStyleSheet(
            checkBoxStyleStr%(absPath(fName_active),absPath(fName_inactive)))
    return checkBox



class ImageProcessorView(QtGui.QWidget):
    _stateChanged = QtCore.pyqtSignal(bool)

    def __init__(self, proc = None):
        super(QtGui.QWidget,self).__init__()

        self.resize(100, 50)
        self.set_processor(proc)
        self.initUI()


    def set_processor(self,proc):
        self.proc = proc


    def initUI(self):
        if self.proc is None:
            return


        # def createStandardCheckbox(parent, img1=None , img2 = None, tooltip = ""):

        vbox = QtGui.QVBoxLayout()

        gridBox = QtGui.QGridLayout()

        self.checkActive = gui_utils.createImageCheckbox(self,
                                                         absPath("images/icon_processor_active"),
                                                         absPath("images/icon_processor_inactive"))


        label = QtGui.QLabel(self.proc.name)

        gridBox.addWidget(self.checkActive,0,0)

        gridBox.addWidget(label,0,1)

        for i, (key, val)  in enumerate(self.proc.kwargs.iteritems()):
            dtype = type(val)
            if dtype == bool:
                check = QtGui.QCheckBox("",self)
                check.stateChanged.connect(self.set_proc_attr_check(check,key,val))
                gridBox.addWidget(QtGui.QLabel(key),i+1,0)
                gridBox.addWidget(check,i+1,1)

            elif dtype in (int,float,np.int,np.float):
                edit = QtGui.QLineEdit(str(val))
                edit.setValidator(QtGui.QDoubleValidator())
                edit.returnPressed.connect(self.set_proc_attr_edit(edit,key,dtype))
                gridBox.addWidget(QtGui.QLabel(key),i+1,0)
                gridBox.addWidget(edit,i+1,1)

        vbox.addLayout(gridBox)
        vbox.addStretch()

        self.setLayout(vbox)
        self.setSizePolicy(Qt.QSizePolicy.Minimum,Qt.QSizePolicy.Minimum)
        self.setStyleSheet("""
        QFrame,QLabel,QLineEdit {
        color: white;
        }
        """)

        self.checkActive.stateChanged.connect(lambda x:self._stateChanged.emit(x!=0))
        # self._stateChanged.connect(self.foo)



    def set_proc_attr_edit(self,obj, key , dtype):
        # print "set proc edit"
        def func():
            # print "setting", key , obj.text()
            setattr(self.proc,key, dtype(obj.text()))
            self._stateChanged.emit(-1)
        return func

    def set_proc_attr_check(self,obj, key , dtype):
        def func():
            # print "setting", key , obj.checkState() !=0
            setattr(self.proc,key, obj.checkState() !=0)
            self._stateChanged.emit(-1)
        return func

    def is_active(self):
        return self.checkActive.checkState() != 0



class ImageProcessorListView(QtGui.QWidget):
    _stateChanged = QtCore.pyqtSignal()
    def __init__(self, imps = []):
        super(QtGui.QWidget,self).__init__()

        self.impViews = [ImageProcessorView(p) for p in imps]

        vbox = QtGui.QVBoxLayout()
        self.gridBox = QtGui.QGridLayout()


        for impView in self.impViews:
            self._include_imp_view(impView)

        vbox.addLayout(self.gridBox)

        vbox.addStretch()

        self.setLayout(vbox)
        self.setSizePolicy(Qt.QSizePolicy.Minimum,Qt.QSizePolicy.Minimum)
        self.setStyleSheet("""
        QFrame,QLabel,QLineEdit {
        color: white;
        }
        """)

    def _include_imp_view(self,impView):
        impView._stateChanged.connect(self.stateChanged)
        # print self.gridBox.rowCount()
        self.gridBox.addWidget(impView,self.gridBox.rowCount(),0)
        # line =  QtGui.QFrame()
        # line.setFrameShape(QtGui.QFrame.HLine)
        # self.mainBox.addWidget(line)


    def add_image_processor(self, imp):
        impView = ImageProcessorView(imp)
        self.impViews.append(impView)
        self._include_imp_view(impView)
        self._stateChanged.emit()


    def stateChanged(self):
        self._stateChanged.emit()



class MainWindow(QtGui.QMainWindow):

    def __init__(self, ):
        super(QtGui.QMainWindow,self).__init__()

        self.resize(300, 200)
        self.setWindowTitle('Test')

        foo  = ImageProcessorListView([
            imageprocessor.BlurProcessor(),
            imageprocessor.FFTProcessor()
        ])

        
        foo.add_image_processor(imageprocessor.BlurProcessor())


        def myfunc(data,para=1.):
            print "myfunc with para", para
            return data*para

        imp = imageprocessor.FuncProcessor(myfunc,"myfunc",para=.1)
        foo.add_image_processor(imp)

        
        self.setCentralWidget(foo)
        self.setStyleSheet("background-color:black;")

    def close(self):
        QtGui.qApp.quit()



if __name__ == '__main__':

    app = QtGui.QApplication(sys.argv)

    win = MainWindow()
    win.show()
    win.raise_()

    sys.exit(app.exec_())


# if __name__ == '__main__':

#     import imgtools


#     ipf = ImageProcessorFlow()


#     ips.add_processor(BlurProcessor())
#     ips.add_processor(CopyProcessor())


#     Z,Y,X = imgtools.ZYX(128)

#     data = 100*np.exp(-100*(X**2+Y**2+Z**2))

#     data += 10.*np.random.normal(0,1.,data.shape)


#     y = ips.apply(data)



"""
Description

some utils functions that are called  from __init__.py

author: Martin Weigert
email: mweigert@mpi-cbg.de
"""

import ConfigParser, StringIO

class MyConfigParser(ConfigParser.ConfigParser):
    def __init__(self,fName = None):
        ConfigParser.ConfigParser.__init__(self)
        self.dummySection = "DUMMY"
        if fName:
            self.read(fName)



    def read(self, fName):
        try:
            text = open(fName).read()
        except IOError:
            raise IOError()
        else:
            file = StringIO.StringIO("[%s]\n%s"%(self.dummySection,text))
            self.readfp(file, fName)

    def get(self,varStr):
        return ConfigParser.ConfigParser.get(self,self.dummySection,varStr)




#!/usr/bin/env python

"""
the actual renderer class to max project 3d data

the modelView and projection matrices are compatible with OpenGL

usage:

rend = VolumeRenderer((400,400))

Nx,Ny,Nz = 200,150,50
d = linspace(0,10000,Nx*Ny*Nz).reshape([Nz,Ny,Nx])

rend.set_data(d)
rend.set_units([1.,1.,.1])
rend.set_projection(projMatPerspective(60,1.,1,10))
rend.set_modelView(dot(transMatReal(0,0,-7),scaleMat(.7,.7,.7)))

out = rend.render()



author: Martin Weigert
email: mweigert@mpi-cbg.de
"""

import logging
logger = logging.getLogger(__name__)



import os


#this is due to some pyinstaller bug!
from scipy.integrate import *


from scipy.misc import imsave
import numpy as np
from scipy.linalg import inv

from time import time


import sys

from gputools import init_device, get_device, OCLProgram, OCLArray, OCLImage
from gputools.core.config import cl_datatype_dict

from spimagine.utils.transform_matrices import *

import spimagine

def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:
        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)

class VolumeRenderer:
    """ renders a data volume by ray casting/max projection

    usage:
               rend = VolumeRenderer((400,400))
               rend.set_data(d)
               rend.set_units(1.,1.,2.)
               rend.set_modelView(rotMatX(.7))
    """

    def __init__(self, size = None):
        """ e.g. size = (300,300)"""

        try:
            # simulate GPU fail...
            # raise Exception()

            # self.dev = OCLDevice(useGPU = True, 
            #                      useDevice = spimagine.__OPENCLDEVICE__)

            #FIXME
            # device should not be reinitialized as then
            # every other queue becomes invalid
            # init_device(useGPU = True,
            #             useDevice = spimagine.config.__OPENCLDEVICE__)
            self.isGPU = True
            self.dtypes = [np.float32,np.uint16]

        except Exception as e:
            print e
            print "could not find GPU OpenCL device -  trying CPU..."

            try:
                init_device(useGPU = False)
                self.isGPU = False
                self.dtypes = [np.float32]
            except Exception as e:
                print e
                print "could not find any OpenCL device ... sorry"

        self.memMax = .7*get_device().get_info("MAX_MEM_ALLOC_SIZE")

        #self.memMax = 2.*get_device().get_info("MAX_MEM_ALLOC_SIZE")

        build_options_basic = ["-I", "%s"%absPath("kernels/"),
                                "-D","maxSteps=%s"%spimagine.config.__DEFAULTMAXSTEPS__]

        self.proc = OCLProgram(absPath("kernels/all_render_kernels.cl"),
                               build_options =
                               build_options_basic +
                               ["-cl-finite-math-only",
                                "-cl-fast-relaxed-math",
                                "-cl-unsafe-math-optimizations",
                                "-cl-mad-enable"])
        try:
            pass

        except Exception as e:

            logger.debug(str(e))
            self.proc = OCLProgram(absPath("kernels/all_render_kernels.cl"),
                                   build_options =
                                   build_options_basic)


        self.invMBuf = OCLArray.empty(16,dtype=np.float32)

        self.invPBuf = OCLArray.empty(16,dtype=np.float32)

        self.projection = np.zeros((4,4))
        self.modelView = np.zeros((4,4))



        if size:
            self.resize(size)
        else:
            self.resize((200,200))

        self.set_dtype()
        self.set_gamma()
        self.set_max_val()
        self.set_min_val()

        self.set_occ_strength(.1)
        self.set_occ_radius(21)
        self.set_occ_n_points(30)

        self.set_alpha_pow()
        self.set_box_boundaries()
        self.set_units()

        self.set_modelView()
        self.set_projection()

    def set_dtype(self,dtype = None):
        if hasattr(self,"dtype") and  dtype is self.dtype:
            return

        if dtype is None:
            dtype = self.dtypes[0]

        if dtype in self.dtypes:
            self.dtype = dtype
        else:
            raise NotImplementedError("data type should be either %s not %s"%(self.dtypes,dtype))

        self.reset_buffer()


    def resize(self,size):
        self.width, self.height = size
        self.reset_buffer()


    def reset_buffer(self):
        self.buf = OCLArray.empty((self.height,self.width),dtype=np.float32)
        self.buf_alpha = OCLArray.empty((self.height, self.width), dtype=np.float32)
        self.buf_depth = OCLArray.empty((self.height, self.width), dtype=np.float32)
        self.buf_normals = OCLArray.empty((self.height,self.width,3),dtype=np.float32)
        self.buf_tmp = OCLArray.empty((self.height,self.width),dtype=np.float32)
        self.buf_tmp_vec = OCLArray.empty((self.height,self.width,3),dtype=np.float32)

        self.buf_occlusion = OCLArray.empty((self.height,self.width),dtype=np.float32)

        self.output = np.zeros((self.height,self.width),dtype=np.float32)
        self.output_alpha = np.zeros((self.height,self.width),dtype=np.float32)
        self.output_depth = np.zeros((self.height,self.width),dtype=np.float32)





    def _get_downsampled_data_slices(self,data):
        """in case data is bigger then gpu texture memory, we should downsample it
        if so returns the slice of data to be rendered
        else returns None (no downsampling)
        """
        # Nstep = int(np.ceil(np.sqrt(1.*data.nbytes/self.memMax)))
        Nstep = int(np.ceil((1.*data.nbytes/self.memMax)**(1./3)))

        slices = [slice(0,d,Nstep) for d in data.shape]
        if Nstep>1:
            logger.info("downsample image by factor of  %s"%Nstep)
            return slices
        else:
            return None

    def set_max_val(self,maxVal = 0.):
        self.maxVal = maxVal

    def set_min_val(self,minVal = 0.):
        self.minVal = minVal
        
    def set_gamma(self,gamma = 1.):
        self.gamma = gamma

    def set_occ_strength(self, occ=.2):
        self.occ_strength = occ

    def set_occ_radius(self, rad =21):
        self.occ_radius = rad

    def set_occ_n_points(self, n_points = 31):
        self.occ_n_points = n_points


    def set_alpha_pow(self,alphaPow = 0.):
        self.alphaPow = alphaPow


    def set_data(self,data, autoConvert = True, copyData = False):
        logger.debug("set_data")

        if not autoConvert and not data.dtype in self.dtypes:
            raise NotImplementedError("data type should be either %s not %s"%(self.dtypes,data.dtype))

        if data.dtype.type in self.dtypes:
            self.set_dtype(data.dtype.type)
            _data = data
        else:
            print "converting type from %s to %s"%(data.dtype.type,self.dtype)
            _data = data.astype(self.dtype,copy = False)

        self.dataSlices = self._get_downsampled_data_slices(_data)

        if self.dataSlices is not None:
            self.set_shape(_data[self.dataSlices].shape[::-1])
        else:
            self.set_shape(_data.shape[::-1])

        t = time()
        self.update_data(_data, copyData = copyData)
        logger.debug("update data: %s ms"%(1000.*(time()-t))) 
        self.update_matrices()

    def set_shape(self,dataShape):
        if self.isGPU:
            self.dataImg = OCLImage.empty(dataShape[::-1],dtype= self.dtype)
        else:
            raise NotImplementedError("TODO")
            # self.dataImg = self.dev.createImage(dataShape,
            #     mem_flags = cl.mem_flags.READ_ONLY,
            #     channel_order = cl.channel_order.INTENSITY,
            #     channel_type = cl_datatype_dict[self.dtype])

        # if self.isGPU:
        #     self.dataImg = self.dev.createImage(dataShape,
        #         mem_flags = cl.mem_flags.READ_ONLY,
        #         channel_type = cl_datatype_dict[self.dtype])
        # else:
        #     self.dataImg = self.dev.createImage(dataShape,
        #         mem_flags = cl.mem_flags.READ_ONLY,
        #         channel_order = cl.channel_order.INTENSITY,
        #         channel_type = cl_datatype_dict[self.dtype])

    def update_data(self,data, copyData = False):
        #do we really want to copy here?
        
        if self.dataSlices is not None:
            self._data = data[self.dataSlices].copy()
        else:
            if copyData:
                self._data = data.copy()
            else:
                self._data = data

        if self._data.dtype != self.dtype:
            self._data = self._data.astype(self.dtype,copy=False)

        self.dataImg.write_array(self._data)

    def set_box_boundaries(self,boxBounds = [-1,1,-1,1,-1,1]):
        self.boxBounds = np.array(boxBounds)

    def set_units(self,stackUnits = np.ones(3)):
        self.stackUnits = np.array(stackUnits)

    def set_projection(self,projection = mat4_perspective()):
        self.projection = projection
        self.update_matrices()

        
    def set_modelView(self, modelView = mat4_identity()):
        self.modelView = 1.*modelView
        self.update_matrices()

    def update_matrices(self):
        if hasattr(self,"dataImg"):
            mScale = self._stack_scale_mat()
            invM = inv(np.dot(self.modelView,mScale))
            self.invMBuf.write_array(invM.flatten().astype(np.float32))
            invP = inv(self.projection)
            self.invPBuf.write_array(invP.flatten().astype(np.float32))


    def _stack_scale_mat(self):
        # scaling the data according to size and units
        Nx,Ny,Nz = self.dataImg.shape
        dx,dy,dz = self.stackUnits

        # mScale =  scaleMat(1.,1.*dx*Nx/dy/Ny,1.*dx*Nx/dz/Nz)
        maxDim = max(d*N for d,N in zip([dx,dy,dz],[Nx,Ny,Nz]))
        return mat4_scale(1.*dx*Nx/maxDim,1.*dy*Ny/maxDim,1.*dz*Nz/maxDim)


    def _render_max_project(self,dtype=np.float32, numParts = 1,currentPart = 0):
        if dtype == np.uint16:
            method = "max_project_short"
        elif dtype == np.float32:
            method = "max_project_float"
        else:
            raise NotImplementedError("wrong dtype: %s",dtype)

        self.proc.run_kernel(method,
                    (self.width,self.height),
                    None,
                    self.buf.data, self.buf_alpha.data,
                             self.buf_depth.data,
                    np.int32(self.width), np.int32(self.height),
                    np.float32(self.boxBounds[0]),
                    np.float32(self.boxBounds[1]),
                             np.float32(self.boxBounds[2]),
                             np.float32(self.boxBounds[3]),
                             np.float32(self.boxBounds[4]),
                             np.float32(self.boxBounds[5]),
                             np.float32(self.minVal),
                             np.float32(self.maxVal),
                             np.float32(self.gamma),
                             np.float32(self.alphaPow),
                             np.int32(numParts),
                             np.int32(currentPart),
                             self.invPBuf.data,
                             self.invMBuf.data,
                             self.dataImg)
        self.output = self.buf.get()
        self.output_alpha = self.buf_alpha.get()
        self.output_depth = self.buf_depth.get()

    def _convolve_scalar(self,buf,radius=11):

        self.proc.run_kernel("conv_x",
                             (self.width,self.height),None,
                             buf.data,
                             self.buf_tmp.data,
                             np.int32(radius))
        self.proc.run_kernel("conv_y",
                             (self.width,self.height),None,
                             self.buf_tmp.data,
                             buf.data ,
                             np.int32(radius))



    def _convolve_vec(self,buf,radius=11):
        self.proc.run_kernel("conv_vec_x",
                             (self.width,self.height),None,
                             buf.data,
                             self.buf_tmp_vec.data,
                             np.int32(radius))

        self.proc.run_kernel("conv_vec_y",
                             (self.width,self.height),None,
                             self.buf_tmp_vec.data,
                             buf.data,
                             np.int32(radius))

    def _render_isosurface2(self):
        self.proc.run_kernel("iso_surface",
                                 (self.width,self.height),
                                 None,
                                 self.buf.data, self.buf_alpha.data,
                                 self.buf_depth.data,self.buf_normals.data,
                                 np.int32(self.width), np.int32(self.height),
                                 np.float32(self.boxBounds[0]),
                                 np.float32(self.boxBounds[1]),
                                 np.float32(self.boxBounds[2]),
                                 np.float32(self.boxBounds[3]),
                                 np.float32(self.boxBounds[4]),
                                 np.float32(self.boxBounds[5]),
                                 np.float32(self.maxVal/2),
                                 np.float32(self.gamma),
                                 self.invPBuf.data,
                                 self.invMBuf.data,
                                 self.dataImg,
                                 np.int32(self.dtype == np.uint16)
                                 )


        self._convolve_vec(self.buf_normals,5)

        self.output = self.buf.get()
        self.output_alpha = self.buf_alpha.get()
        self.output_depth = self.buf_depth.get()
        self.output_normals = self.buf_normals.get()

    def _render_isosurface(self):
        """
        with ambient occlusion
        """

        self.proc.run_kernel("iso_surface",
                                 (self.width,self.height),
                                 None,
                                 self.buf.data, self.buf_alpha.data,
                                 self.buf_depth.data,self.buf_normals.data,
                                 np.int32(self.width), np.int32(self.height),
                                 np.float32(self.boxBounds[0]),
                                 np.float32(self.boxBounds[1]),
                                 np.float32(self.boxBounds[2]),
                                 np.float32(self.boxBounds[3]),
                                 np.float32(self.boxBounds[4]),
                                 np.float32(self.boxBounds[5]),
                                 np.float32(self.maxVal/2),
                                 np.float32(self.gamma),
                                 self.invPBuf.data,
                                 self.invMBuf.data,
                                 self.dataImg,
                                 np.int32(self.dtype == np.uint16)
                                 )
        self._convolve_vec(self.buf_normals,7)


        self.proc.run_kernel("occlusion",
                                 (self.width,self.height),
                                 None,
                                 self.buf_occlusion.data,
                                 np.int32(self.width), np.int32(self.height),
                                 np.int32(self.occ_radius),
                                 np.int32(self.occ_n_points),
                             self.buf_depth.data,
                             self.buf_normals.data,
                                 )

        self._convolve_scalar(self.buf_occlusion,5)
        #self._convolve_vec(self.buf_normals,5)

        self.proc.run_kernel("shading",
                                 (self.width,self.height),
                                 None,
                                 self.buf.data, self.buf_alpha.data,
                                 np.int32(self.width), np.int32(self.height),
                                 self.invPBuf.data,
                                 self.invMBuf.data,
                                np.float32(self.occ_strength),
                                 self.buf_normals.data,
                                self.buf_depth.data,
                                self.buf_occlusion.data,

                             )


        #self._convolve_scalar(self.buf,13)
        #self._convolve_vec(self.buf_normals,101)

        self.output = self.buf.get()
        self.output_alpha = self.buf_alpha.get()
        self.output_depth = self.buf_depth.get()
        self.output_normals = self.buf_normals.get()
        self.output_occlusion = self.buf_occlusion.get()



    def render(self,data = None, stackUnits = None,
               minVal = None, maxVal = None, gamma = None,
               modelView = None, projection = None,
               boxBounds = None, return_alpha = False, method="max_project",
               numParts = 1, currentPart = 0):

        if data is not None:
            self.set_data(data)

        if maxVal is not None:
            self.set_max_val(maxVal)

        if minVal is not None:
            self.set_min_val(minVal)

        if gamma is not None:
            self.set_gamma(gamma)

        if stackUnits is not None:
            self.set_units(stackUnits)

        if modelView is not None:
            self.set_modelView(modelView)

        if projection is not None:
            self.set_projection(projection)

        if not hasattr(self,'dataImg'):
            print "no data provided, set_data(data) before"
            return

        if  modelView is None and not hasattr(self,'modelView'):
            print "no modelView provided and set_modelView() not called before!"
            return

        if method=="max_project":
            self._render_max_project(self.dtype,numParts,currentPart)

        if method=="iso_surface":
            self._render_isosurface()



def renderSpimFolder(fName, outName,width, height, start =0, count =-1,
                     rot = 0, isStackScale = True):
    """legacy"""

    rend = VolumeRenderer((500,500))
    for t in range(start,start+count):
        print "%i/%i"%(t+1,count)
        modelView = scaleMat()
        modelView =  dot(rotMatX(t*rot),modelView)
        modelView =  dot(transMat(0,0,4),modelView)

        rend.set_dataFromFolder(fName,pos=start+t)

        rend.set_modelView(modelView)
        out = rend.render(scale = 1200, density=.01,gamma=2,
                          isStackScale = isStackScale)

        imsave("%s_%s.png"%(outName,str(t+1).zfill(int(ceil(log10(count+1))))),out)




# two test functions to get the ray coordinates in the kernel...
def _getOrig(P,M,u=1,v=0):
    orig0 = dot(inv(P),[u,v,-1,1])
    orig0 = dot(inv(M),orig0)
    orig0 = orig0/orig0[-1]
    return orig0
def _getDirec(P,M,u=1,v=0):
    direc0 = dot(inv(P),[u,v,1,1])
    direc0 = direc0/direc0[-1];orig0 = dot(inv(P),[u,v,-1,1]);
    direc0 = direc0 - orig0; direc0 = direc0/norm(direc0)
    return dot(inv(M),direc0)


def test_simple():
    import pylab
    
    N= 64
    d = np.linspace(0,1,N**3).reshape((N,)*3).astype(np.float32)

    rend = VolumeRenderer((400,400))

    rend.set_data(d)
    rend.render()
    out = rend.output
    pylab.imshow(out)
    pylab.show()

def test_simple2():
    import time

    N = 64

    x = np.linspace(-1,1,N)
    Z,Y,X = np.meshgrid(x,x,x,indexing="ij")
    R = np.sqrt(X**2+Y**2+Z**2)

    
    d = 10000*np.exp(-10*R**2)

    rend = VolumeRenderer((600,600))

    # rend.set_modelView(mat4_rotation(.5,0,1.,0))
    rend.set_modelView(mat4_translate(0,0,-10.))

    # rend.set_box_boundaries(.3*np.array([-1,1,-1,1,-1,1]))
    t1 = time.time()

    rend.dev.queue.finish()
    rend.set_data(d, autoConvert = True)
    rend.dev.queue.finish()

    t2 = time.time()

    rend.dev.queue.finish()
    rend.render(maxVal = 10000.)
    out =  rend.output
    rend.dev.queue.finish()

    print "time to set data %s^3:\t %.2f ms"%(N,1000*(t2-t1))

    print "time to render %s^3:\t %.2f ms"%(N,1000*(time.time()-t2))

    return d, rend, out

def test_new_iso():
    import time

    N = 128

    x = np.linspace(-1,1,N)
    Z,Y,X = np.meshgrid(x,x,x,indexing="ij")
    R = np.sqrt(X**2+Y**2+Z**2)

    
    d = 10000*np.exp(-10*R**2)
    d += 00*np.random.uniform(-1,1,d.shape) 

    rend = VolumeRenderer((600,600))

    # rend.set_modelView(mat4_rotation(.5,0,1.,0))
    rend.set_modelView(mat4_translate(0,0,-1.))

    # rend.set_box_boundaries(.3*np.array([-1,1,-1,1,-1,1]))
    t1 = time.time()

    rend.dev.queue.finish()
    rend.set_data(d, autoConvert = True)
    rend.dev.queue.finish()

    t2 = time.time()

    rend.dev.queue.finish()
    rend.render(maxVal = 1000., method="iso_surface_new")
    out = rend.output
    rend.dev.queue.finish()

    print "time to set data %s^3:\t %.2f ms"%(N,1000*(t2-t1))

    print "time to render %s^3:\t %.2f ms"%(N,1000*(time.time()-t2))

    return d, rend, out

def test_real():
    import imgtools
    import time

    d = imgtools.read3dTiff("/Users/mweigert/Data/sqeazy_corpus/Norden_GFP-LAP_4-1.tif")

    rend = VolumeRenderer((600,600))

    rend.set_modelView(mat4_rotation(.5,0,1.,0))

    # rend.set_box_boundaries(.3*np.array([-1,1,-1,1,-1,1]))
    t1 = time.time()

    rend.set_data(d, autoConvert = True)
    rend.set_units([1.,1.,6.])
    t2 = time.time()

    rend.render(maxVal = 200.)

    print "time to set data :\t %.2f ms"%(1000*(t2-t1))

    print "time to render:\t %.2f ms"%(1000*(time.time()-t2))

    return d, rend, rend.output


def test_speed(N=128,renderWidth = 400, numParts = 1):
    import time

    d = np.ones((N,)*3,dtype=np.float32)
    rend = VolumeRenderer((renderWidth,)*2)


    # rend.set_box_boundaries(.3*np.array([-1,1,-1,1,-1,1]))
    t1 = time.time()

    rend.dev.queue.finish()
    rend.set_data(d, autoConvert = True)
    rend.dev.queue.finish()

    rend.set_modelView(mat4_rotation(.5,0,1.,0))

    t2 = time.time()
    rend.dev.queue.finish()
    for i in range(10):
        rend.render(method = "max_project_part", maxVal = 200.,
                          currentPart=0,numParts=numParts)
    rend.dev.queue.finish()

    t3 = time.time()

    print "time to set data %s^3:\t %.2f ms"%(N,1000.*(t2-t1))

    print "time to render %s^3:\t %.2f ms"%(N,1000./10.*(t3-t2))

    return 1000./10.*(t3-t2)

    
if __name__ == "__main__":
    from time import time
    # test_simple()
    # test_speed(256)
    from gputools.utils.utils import remove_cache_dir, get_cache_dir
    remove_cache_dir()


    N = 128

    x = np.linspace(-1,1,N)
    #R1 = np.sqrt(np.sum([(_X-.2)**2 for _X in np.meshgrid(x,x,x,indexing="ij")],axis=0))
    #R2 = np.sqrt(np.sum([(_X+.2)**2 for _X in np.meshgrid(x,x,x,indexing="ij")],axis=0))

    Z,Y,X = np.meshgrid(x,x,x,indexing="ij")
    R1 = np.sqrt((X-.2)**2+Y**2+Z**2)
    R2 = np.sqrt((X+.2)**2+Y**2+Z**2)

    d = np.exp(-30*R1**2)+ np.exp(-30*R2**2)
    #d += .03*np.random.uniform(0.,1.,d.shape)

    rend = VolumeRenderer((400,400))


    rend.set_modelView(np.dot(mat4_translate(0,0,-2.),mat4_rotation(0.,0.,1.,0.)))


    rend.set_data(d.astype(np.float32))

    t = time()
    #rend.render(maxVal = .5, method = "iso_surface")
    rend.render(maxVal = .5)
    print time()-t
    out = rend.output_normals[...,0]
    out = rend.output_occlusion
    #out[rend.output_depth>10] = 0.6
    #out = rend.output_depth

    import pylab
    pylab.figure(1)
    pylab.clf()
    ss = (slice(100,300),slice(120,280))
    #pylab.imshow(out[140:260,140:260])
    pylab.imshow(out[ss])

    pylab.show()



import logging
logger = logging.getLogger(__name__)

import ConfigParser
import StringIO


class MyConfigParser(ConfigParser.SafeConfigParser):
    def __init__(self,fName = None, defaults = {}):
        ConfigParser.SafeConfigParser.__init__(self,defaults)
        self.dummySection = "DUMMY"
        if fName:
            self.read(fName)

    def read(self, fName):
        try:
            text = open(fName).read()
        except IOError:
            print "could not open %s"%fName
        else:
            file = StringIO.StringIO("[%s]\n%s"%(self.dummySection,text))
            self.readfp(file, fName)

    def get(self,key, defaultValue = None):
        try:
            val =  ConfigParser.ConfigParser.get(self,self.dummySection,key)
            logger.debug("from config file: %s = %s "%(key,val))
            return val
        except Exception as e:
            logger.debug("%s (%s)"%(e,key))
            return defaultValue



__version__ = None

try:
    from pkg_resources import get_distribution, DistributionNotFound
    __version__ = get_distribution("spimagine").version
except Exception as e:
    print e


import logging
logging.basicConfig(format='%(levelname)s:%(name)s | %(message)s')
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
#logger.setLevel(logging.DEBUG)


from spimagine.config import config

from spimagine.models.data_model import DataModel,DemoData, SpimData, TiffData, CZIData, TiffFolderData, NumpyData

from spimagine.utils.imgutils import read3dTiff, write3dTiff

from spimagine.gui.volshow import volshow, volfig, qt_exec

from spimagine.gui.mesh import Mesh, SphericalMesh, EllipsoidMesh

from spimagine.utils import *


import numpy as np

class Quaternion():
    def __init__(self,w=1.,x=0.,y=0.,z=0.):
        
        self.data = np.array([w,x,y,z])

    @classmethod
    def copy(cls,rhs):
        return Quaternion(*rhs.data)


    def __getitem__(self,i):
        return self.data[i]

    def __setitem__(self,i,val):
        self.data[i] = val



    def conj(self):
        return Quaternion(self[0],-self[1],-self[2],-self[2])

    def norm(self):
        return np.linalg.norm(self.data)

    def __add__(self,q):
        return Quaternion(*(self.data+q.data))

    def __sub__(self,q):
        return Quaternion(*(self.data-q.data))


    def __mul__(self,q):
        if isinstance(q,Quaternion):
            a1,b1,c1,d1 = self.data
            a2,b2,c2,d2 = q.data
            return Quaternion(a1*a2 - b1*b2 - c1*c2 - d1*d2,
                          a1*b2 + b1*a2 + c1*d2 - d1*c2,
                          a1*c2 - b1*d2 + c1*a2 + d1*b2,
                          a1*d2 + b1*c2 - c1*b2 + d1*a2)
        else:
            return Quaternion(*(q*self.data))

    def __repr__(self):
        return "Quaternion(%s,%s,%s,%s)"%tuple(self.data)

    def dot(self,q):
        return np.inner(self.data,q.data)

    def normalize(self):
        return Quaternion(*(self.data*1./self.norm()))

    def toRotation4(self):
        a,b,c,d = self.data
        return np.array([
    [a**2+b**2-c**2-d**2, 2*(b*c-a*d), 2*(b*d+a*c),0],
    [2*(b*c+a*d), a**2-b**2+c**2-d**2, 2*(c*d-a*b),0],
    [2*(b*d-a*c), 2*(c*d+a*b),  a**2-b**2-c**2+d**2,0],
    [0,0,0,1]
    ])

    def toRotation3(self):
        a,b,c,d = self.data
        return np.array([
    [a**2+b**2-c**2-d**2, 2*(b*c-a*d), 2*(b*d+a*c)],
    [2*(b*c+a*d), a**2-b**2+c**2-d**2, 2*(c*d-a*b)],
    [2*(b*d-a*c), 2*(c*d+a*b),  a**2-b**2-c**2+d**2],
    ])



def quaternion_slerp(q1,q2,t):
    w = np.arccos(q1.dot(q2))
    if w<1.e-10:
        return Quaternion.copy(q1)
    return (q1*(np.sin((1.-t)*w)/np.sin(w)))+q2*(np.sin(t*w)/np.sin(w))

if __name__ == '__main__':
    q1 = Quaternion(1,0,0,0)
    q2 = Quaternion(0,1,0,0)

    for t in np.linspace(0,1,10):
        q = quaternion_slerp(q1,q2,t)
        print t, q, q.norm()



#!/usr/bin/env python

"""
main gui program that creates the frame with the rendering controls.

Drag and Drop into rendering canvas is supported for
- Tiff files
- BScope Spim Data Folders (Myers Lab)

and might be extended by writing a corressponding DataModel (defined in data_model.py)
for it


author: Martin Weigert
email: mweigert@mpi-cbg.de
"""


import sys
import os


from PyQt4 import QtGui, QtCore

import argparse

import logging
logger = logging.getLogger(__name__)

def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    import sys

    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        logger.debug("found MEIPASS: %s "%os.path.join(base_path, os.path.basename(myPath)))

        return os.path.join(base_path, os.path.basename(myPath))
    except Exception as e:
        logger.debug("did not find MEIPASS: %s "%e)


        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)


    
def main():

    parser = argparse.ArgumentParser(description='spimagine rendering application ')
    parser.add_argument('fname',
                        type=str,
                        nargs='*',
                        help='the files/folder to open (e.g. tif, folder of tif) ',
                        default = None)
    
    parser.add_argument('-p',
                        dest='prefetch',
                        type = int,
                        default=0,
                        help='prefetch size (should not be negative, e.g. -p 2)')

    parser.add_argument('-D',
                        action='store_true',
                        help = "output DEBUG messages")

    try:
        args = parser.parse_args()
    except:
        parser.print_help()
        sys.exit(0)

        
    if args.D:
        logger = logging.getLogger("spimagine")
        logger.setLevel(logging.DEBUG)

    app = QtGui.QApplication(sys.argv)


    #splash screen
    # print 'FIIIIIIIIIIII'
    # print absPath('../gui/images/splash.png')
    # print os.listdir(absPath("."))
    # print sys._MEIPASS

    pixmap = QtGui.QPixmap(absPath('../gui/images/splash.png'))
    splash = QtGui.QSplashScreen(pixmap, QtCore.Qt.WindowStaysOnTopHint)
    splash.setMask(pixmap.mask())
    splash.show()
    app.processEvents()


    from spimagine.gui.mainwidget import MainWidget
    from spimagine.models.data_model import DemoData, DataModel
        


    
    app.setWindowIcon(QtGui.QIcon(absPath('../gui/images/spimagine.png')))

    if sys.platform.startswith("win"):
    	QtGui.QApplication.setStyle(QtGui.QStyleFactory.create("CleanLooks"))


    win = MainWidget()
    if args.fname:
        if len(args.fname)==1:
            win.setModel(DataModel.fromPath(args.fname[0]))
        else:
            win.setModel(DataModel.fromPath(args.fname))
    else:
        win.setModel(DataModel(DemoData()))

    win.show()
    win.raise_()

    splash.finish(win)

    sys.exit(app.exec_())


if __name__ == '__main__':
    main()    



import logging
logger = logging.getLogger(__name__)


import os
import numpy as np
import re
from PIL import Image


import warnings 

with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    from spimagine.lib.tifffile import TiffFile, imsave, imread
    from spimagine.lib.czifile import CziFile



def _read3dTiff_PIL(fName):
    img = Image.open(fName)
    i = 0
    data = []
    while True:
        try:
            img.seek(i)
        except EOFError:
            break
        data.append(np.asarray(img))
        i += 1

    return np.array(data)

def read3dTiff(fName):
    return imread(fName)


def write3dTiff(data,fName):
    imsave(fName,data)

        
def getTiffSize(fName):
    img = Image.open(fName, 'r')
    depth = 0
    while True:
        try:
            img.seek(depth)
        except Exception as e:
            break
        depth += 1

    return (depth,)+img.size[::-1]


def readCziFile(fName):
    with CziFile(fName)  as f:
        return np.squeeze(f.asarray())
            


def parseIndexFile(fname):
    """
    returns (t,z,y,z) dimensions of a spim stack
    """
    try:
        lines = open(fname).readlines()
    except IOError:
        print "could not open and read ",fname
        return None

    items = lines[0].replace("\t",",").split(",")
    try:
        stackSize = [int(i) for i in items[-4:-1]] +[len(lines)]
    except Exception as e:
        print e
        print "couldnt parse ", fname
        return None
    stackSize.reverse()
    return stackSize


def parseMetaFile(fName):
    """
    returns pixelSizes (dx,dy,dz)
    """

    with open(fName) as f:
        s = f.read()
        try:
            z1 = np.float(re.findall("StartZ.*",s)[0].split("\t")[2])
            z2 = np.float(re.findall("StopZ.*",s)[0].split("\t")[2])
            zN = np.float(re.findall("NumberOfPlanes.*",s)[0].split("\t")[2])

            return (.162,.162, (1.*z2-z1)/zN)
        except Exception as e:
            print e
            print "coulndt parse ", fName
            return (1.,1.,1.)



def fromSpimFolder(fName,dataFileName="data/data.bin",indexFileName="data/index.txt",pos=0,count=1):
    stackSize = parseIndexFile(os.path.join(fName,indexFileName))

    if stackSize:
        # clamp to pos to stackSize
        pos = min(pos,stackSize[0]-1)
        pos = max(pos,0)

        if count>0:
            stackSize[0] = min(count,stackSize[0]-pos)
        else:
            stackSize[0] = max(0,stackSize[0]-pos)

        with open(os.path.join(fName,dataFileName),"rb") as f:
            f.seek(2*pos*np.prod(stackSize[1:]))
            return np.fromfile(f,dtype="<u2",
                               count=np.prod(stackSize)).reshape(stackSize)

            t = time()
            ds.append(func(fName))
            print "%s\ntime: %.2f ms"%(func.__name__, 1000.*(time()-t))

        assert np.allclose(*ds)

        
def test_czi():
    d = readCziFile("test_data/retina.czi")
    return d
    
if __name__ == '__main__':

    # test_tiff()

    d = test_czi()



import os
import numpy as np
from PyQt4 import QtCore
import time
import Queue
# import h5py
import SpimUtils
import re
from PIL import Image
from collections import defaultdict


def getTiffSize(fName):
    img = Image.open(fName, 'r')
    depth = 0
    while True:
        try:
            img.seek(depth)
        except Exception as e:
            break
        depth += 1

    return (depth,)+img.size[::-1]


def read3dTiff(fName, depth = -1, dtype = np.uint16):

    if not depth>0:
        depth = getTiffSize(fName)[0]

    img = Image.open(fName, 'r')
    stackSize = (depth,) + img.size[::-1]

    data = np.empty(stackSize,dtype=dtype)

    for i in range(stackSize[0]):
        img.seek(i)
        data[i,...] = np.asarray(img, dtype= dtype)

    return data




class GenericData():
    dataFileError = Exception("not a valid file")
    def __init__(self):
        self.stackSize = None
        self.stackUnits = None

    def sizeT(self):
        return None

    def __getitem__(self,int):
        return None


class SpimData(GenericData):
    def __init__(self,fName = ""):
        GenericData.__init__(self)
        self.load(fName)

    def load(self,fName):
        if fName:
            try:
                self.stackSize = SpimUtils.parseIndexFile(os.path.join(fName,"data/index.txt"))
                self.stackUnits = SpimUtils.parseMetaFile(os.path.join(fName,"metadata.txt"))
                self.fName = fName
            except Exception as e:
                print e
                self.fName = ""
                raise Exception("couldnt open %s as SpimData"%fName)

            try:
                # try to figure out the dimension of the dark frame stack
                darkSizeZ = os.path.getsize(os.path.join(self.fName,"data/darkstack.bin"))/2/self.stackSize[2]/self.stackSize[3]
                print darkSizeZ
                with open(os.path.join(self.fName,"data/darkstack.bin"),"rb") as f:
                    self.darkStack = np.fromfile(f,dtype="<u2").reshape([darkSizeZ,self.stackSize[2],self.stackSize[3]])

            except Exception as e:
                print e
                print "couldn't find darkstack"
                

    def sizeT(self):
        if self.fName:
            return self.stackSize[0]
        else:
            return 0

    def __getitem__(self,pos):
        if self.stackSize and self.fName:
            if pos<0 or pos>=self.stackSize[0]:
                raise IndexError("0 <= pos <= %i, but pos = %i"%(self.stackSize[0]-1,pos))


            pos = max(0,min(pos,self.stackSize[0]-1))
            voxels = np.prod(self.stackSize[1:])
            # use int64 for bigger files
            offset = np.int64(2)*pos*voxels

            with open(os.path.join(self.fName,"data/data.bin"),"rb") as f:
                f.seek(offset)
                return np.fromfile(f,dtype="<u2",
                count=voxels).reshape(self.stackSize[1:])
        else:
            return None


class TiffData(GenericData):
    def __init__(self,fName = ""):
        GenericData.__init__(self)
        self.load(fName)

    def load(self,fName, stackUnits = None):
        if fName:
            try:
                self.stackSize = (1,)+ getTiffSize(fName)
            except Exception as e:
                print e
                self.fName = ""
                raise Exception("couldnt open %s as TiffData"%fName)
                return

            if stackUnits:
                self.stackUnits = stackUnits
            self.fName = fName

    def sizeT(self):
        if self.fName:
            return 1
        else:
            return 0

    def __getitem__(self,pos):
        if self.stackSize and self.fName:
            return read3dTiff(self.fName,depth = self.stackSize[1])
        else:
            return None


def test_spimLoader():
    fName = "/Users/mweigert/Data/SIM/SIM_Hela3_18/"

    data = SpimData(fName)


    dt = 0

    for i in range(20):
        print i
        time.sleep(.00001)
        t = time.time()
        d = loader[i]
        dt += (time.time()-t)

    print "%.4s per fetch "%(dt/20.)

    loader.stop()



    
if __name__ == '__main__':


    fName = "/Users/mweigert/Data/SIM/DarkTest"

    data = SpimData(fName)



#!/usr/bin/env python

"""

compatible with OpenGL, i.e.

mat4_perspective = gluPerspective
...etc


author: Martin Weigert
email: mweigert@mpi-cbg.de
"""

import numpy as np
from quaternion import Quaternion





def mat4_scale(x =1.,y=1.,z=1.):
    return np.array([x, 0.0, 0.0, 0.,
                  0.0, y, 0.0, 0.0,
                  0.0, 0.0, z, 0.0,
                  0, 0, 0, 1.0]).reshape(4,4)



def mat4_rotation(w=0,x=1,y=0,z=0):
    n = np.array([x,y,z])
    n *= 1./np.sqrt(1.*np.sum(n**2))
    q = Quaternion(np.cos(.5*w),*(np.sin(.5*w)*n))
    return q.toRotation4()



def mat4_perspective(fovy = 45,aspect = 1.,
                       z1 = 0.1, z2 = 10):
    """ like gluPerspective(fovy, aspect, zNear, zFar)
        fovy in degrees
    """
    f = 1./np.tan(fovy/180.*np.pi/2.)
    return np.array([[1.*f/aspect,0,0,0],
                  [0,f,0,0],
                  [0,0,-1.*(z2+z1)/(z2-z1),-2.*z1*z2/(z2-z1)],
                  [0,0,-1,0]])

def mat4_stereo_perspective(fovy = 45,aspect = 1.,
                            z1 = 0.1, z2 = 10, eye_shift = 0):
    h = z1 * np.tan(fovy/180.*np.pi/2.)
    w = h * aspect;

    return mat4_frustrum(-w-eye_shift,w-eye_shift,-h,h,z1,z2)
    

def mat4_frustrum(left, right, bottom, top, zNear, zFar):
    return np.array([[2.*zNear/(right-left),0,1.*(right+left)/(right-left),0],
                     [0,2.*zNear/(top-bottom),(top+bottom)/(top-bottom),0],
                     [0,0,-1.*(zFar+zNear)/(zFar-zNear),-2.*zFar*zNear/(zFar-zNear)],
                     [0,0,-1.,0]])


def mat4_ortho(x1 = -1, x2 = 1,
                 y1 = -1, y2 = 1,
                 z1 = -1, z2 = 1):
    """ like glOrtho """
    ax,bx = x2+x1,x2-x1
    ay,by = y2+y1,y2-y1
    az,bz = z2+z1,z2-z1
    return np.array([[2./bx,0,0,-1.*ax/bx],
                     [0,2./by,0,-1.*ay/by],
                     [0,0,-2./bz,-1.*az/bz],
                     [0,0,0,1.]])


def mat4_identity():
    return np.identity(4)


def mat4_translate(x=0,y=0,z=0):
    M = mat4_identity()
    M[:3,3] = x,y,z
    return M

def mat4_lookat(eye,center,up):
    _eye = np.array(eye)
    _center = np.array(center)
    _up = np.array(up)

    _fwd = _center - _eye

    #normalize
    _fwd *= 1./np.sqrt(np.sum(_fwd**2))

    
    s = np.cross(_fwd, _up)

    s *= 1./np.sqrt(np.sum(s**2))

    
    _up = np.cross(s, _fwd)

    M = np.identity(4)
    M[0,:3] = s
    M[1,:3] = _up
    M[2,:3] = - _fwd

    return np.dot(M,mat4_translate(*(-_eye)))




# gluLookAt(GLdouble eyex, GLdouble eyey, GLdouble eyez, GLdouble centerx,
#       GLdouble centery, GLdouble centerz, GLdouble upx, GLdouble upy,
#       GLdouble upz)
# {
#     float forward[3], side[3], up[3];
#     GLfloat m[4][4];

#     forward[0] = centerx - eyex;
#     forward[1] = centery - eyey;
#     forward[2] = centerz - eyez;

#     up[0] = upx;
#     up[1] = upy;
#     up[2] = upz;

#     normalize(forward);

#     /* Side = forward x up */
#     cross(forward, up, side);
#     normalize(side);

#     /* Recompute up as: up = side x forward */
#     cross(side, forward, up);

#     __gluMakeIdentityf(&m[0][0]);
#     m[0][0] = side[0];
#     m[1][0] = side[1];
#     m[2][0] = side[2];

#     m[0][1] = up[0];
#     m[1][1] = up[1];
#     m[2][1] = up[2];

#     m[0][2] = -forward[0];
#     m[1][2] = -forward[1];
#     m[2][2] = -forward[2];

#     glMultMatrixf(&m[0][0]);
#     glTranslated(-eyex, -eyey, -eyez);
# }


if __name__ == '__main__':

    print mat4_lookat([0,0,10],[0,0,0],[0,1,0])


    # print rotMat(.1,0,0,1)
    # orthoM = projMatOrtho(-2,2,-2,2,-10,10)

    # perspM = projMatPerspective(45,1,.1,10)

    # x  = np.dot(orthoM,[0,0,0,1])
    # print x


    # x  = np.dot(perspM,[0,0,.1,1])
    # print x

    # print orthoM




import logging
logger = logging.getLogger(__name__)


import sys
import os
from PyQt4 import QtCore
from PyQt4 import QtGui
from PyQt4 import QtOpenGL



import numpy as np


from gui_utils import createImageCheckbox,createStandardCheckbox, createStandardButton
from spimagine.gui.floatslider import FloatSlider

import spimagine


def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:
        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)


checkBoxStyleStr = """
    QCheckBox::indicator:checked {
    background:black;
    border-image: url(%s);}
    QCheckBox::indicator:unchecked {
    background:black;
    border-image: url(%s);}
    """

def createImgCheckBox(fName_active,fName_inactive):
    checkBox = QtGui.QCheckBox()
    checkBox.setStyleSheet(
            checkBoxStyleStr%(absPath(fName_active),absPath(fName_inactive)))
    return checkBox



class VolumeSettingsPanel(QtGui.QWidget):
    _stackUnitsChanged = QtCore.pyqtSignal(float,float,float)
    _boundsChanged =  QtCore.pyqtSignal(float,float,float,float,float,float)
    _alphaPowChanged = QtCore.pyqtSignal(float)
    _rgbColorChanged = QtCore.pyqtSignal(float, float,float)
    
    def __init__(self):
        super(QtGui.QWidget,self).__init__()

        self.resize(50, 300)
        self.initUI()


    def initUI(self):



        vbox = QtGui.QVBoxLayout()

        vbox.addWidget(QtGui.QLabel("Stack units",alignment = QtCore.Qt.AlignCenter))
        # The stack units line edits
        stackLabels = ["x","y","z"]


        self.stackEdits = []
        for lab in stackLabels:
            hbox = QtGui.QHBoxLayout()
            edit = QtGui.QLineEdit("1.0")
            edit.setValidator(QtGui.QDoubleValidator(bottom=1e-10))
            edit.returnPressed.connect(self.stackUnitsChanged)
            hbox.addWidget(QtGui.QLabel(lab))
            hbox.addWidget(edit)
            vbox.addLayout(hbox)
            self.stackEdits.append(edit)


        vbox.addWidget(QtGui.QLabel("Borders",alignment = QtCore.Qt.AlignCenter))

        gridBox = QtGui.QGridLayout()
        self.sliderBounds = [QtGui.QSlider(QtCore.Qt.Horizontal) for _ in range(6)]
        for i,s in enumerate(self.sliderBounds):
            s.setTickPosition(QtGui.QSlider.TicksBothSides)
            s.setRange(-100,100)
            s.setTickInterval(1)
            s.setFocusPolicy(QtCore.Qt.ClickFocus)
            s.setTracking(True)
            s.setValue(-100+200*(i%2))
            s.valueChanged.connect(self.boundsChanged)
            s.setStyleSheet("height: 18px; border = 0px;")

            gridBox.addWidget(s,i,1)


        vbox.addLayout(gridBox)
            
        line =  QtGui.QFrame()
        line.setFrameShape(QtGui.QFrame.HLine)

        vbox.addWidget(line)
        vbox.addWidget(QtGui.QLabel("Display",alignment = QtCore.Qt.AlignCenter))

        # the perspective/box checkboxes
        self.checkProj = createImageCheckbox(self, absPath("images/rays_persp.png"),
                                             absPath("images/rays_ortho.png"), tooltip="projection")

        self.checkBox = createImageCheckbox(self, absPath("images/wire_cube.png"),
                                            absPath("images/wire_cube_incative.png"),
                                            tooltip="toggle box")

        self.checkInvert = createStandardCheckbox(self,
                                               tooltip="invert colors")

        self.butColor = createStandardButton(self,absPath("images/icon_colors.png"),
                                             method = self.onButtonColor,
                                                    tooltip="color")

        gridBox = QtGui.QGridLayout()

        gridBox.addWidget(QtGui.QLabel("projection:\t"),1,0)
        gridBox.addWidget(self.checkProj,1,1)

        gridBox.addWidget(QtGui.QLabel("bounding box:\t"),2,0)
        gridBox.addWidget(self.checkBox,2,1)

        gridBox.addWidget(QtGui.QLabel("invert colors:\t"),3,0)
        gridBox.addWidget(self.checkInvert,3,1)

        gridBox.addWidget(QtGui.QLabel("colormap:\t"),4,0)

        self.colorCombo = QtGui.QComboBox()

        self.colormaps = list(spimagine.config.__COLORMAPDICT__.keys())

        self.colorCombo.setIconSize(QtCore.QSize(100,20))

        for s in self.colormaps:
            self.colorCombo.addItem(QtGui.QIcon(absPath("../colormaps/cmap_%s.png"%s)),"")

        gridBox.addWidget(self.colorCombo,4,1)

        gridBox.addWidget(self.butColor,5,0)


        self.sliderAlphaPow = FloatSlider(QtCore.Qt.Horizontal)
        self.sliderAlphaPow.setRange(0,2.,100)
        self.sliderAlphaPow.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.sliderAlphaPow.setTracking(True)
        self.sliderAlphaPow.setValue(1.)

        gridBox.addWidget(QtGui.QLabel("opacity transfer:\t"),6,0)
        gridBox.addWidget(self.sliderAlphaPow,6,1)

        self.sliderOcc = FloatSlider(QtCore.Qt.Horizontal)
        self.sliderOcc.setRange(0,1.,100)
        self.sliderOcc.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.sliderOcc.setTracking(True)
        self.sliderOcc.setValue(.1)

        gridBox.addWidget(QtGui.QLabel("AO strength:\t"),7,0)
        gridBox.addWidget(self.sliderOcc,7,1)

        self.sliderOccRadius = FloatSlider(QtCore.Qt.Horizontal)
        self.sliderOccRadius.setRange(4.,100.,100)
        self.sliderOccRadius.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.sliderOccRadius.setTracking(True)
        self.sliderOccRadius.setValue(21)

        gridBox.addWidget(QtGui.QLabel("AO radius :\t"),8,0)
        gridBox.addWidget(self.sliderOccRadius,8,1)

        self.sliderOccNPoints = FloatSlider(QtCore.Qt.Horizontal)
        self.sliderOccNPoints.setRange(10.,200.,100)
        self.sliderOccNPoints.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.sliderOccNPoints.setTracking(True)
        self.sliderOccNPoints.setValue(31)

        gridBox.addWidget(QtGui.QLabel("AO n points:\t"),9,0)
        gridBox.addWidget(self.sliderOccNPoints,9,1)

        vbox.addLayout(gridBox)

        # vbox.addStretch()
        line =  QtGui.QFrame()
        line.setFrameShape(QtGui.QFrame.HLine)

        vbox.addWidget(line)

        #################

        line =  QtGui.QFrame()
        line.setFrameShape(QtGui.QFrame.HLine)
        vbox.addWidget(line)

        self.dimensionLabel = QtGui.QLabel("Dimensions:",alignment = QtCore.Qt.AlignLeft)
        vbox.addWidget(self.dimensionLabel)

        self.statsLabel = QtGui.QLabel("Max: Min: Mean:",alignment = QtCore.Qt.AlignLeft)
        vbox.addWidget(self.statsLabel)

        self.setStyleSheet("""
        QFrame,QLabel,QLineEdit {
        color: white;
        }
        """)
        self.colorCombo.setStyleSheet("background-color:none;")

        vbox.addStretch()

        self.setLayout(vbox)



    def onButtonColor(self):
        col = QtGui.QColorDialog.getColor()

        if col.isValid():
            color = 1./255*np.array(col.getRgb()[:3])
            self._rgbColorChanged.emit(*color)

    def setStackUnits(self,px,py,pz):
        for e,p in zip(self.stackEdits,[px,py,pz]):
            e.setText(str(p))

    def setBounds(self,x1,x2,y1,y2,z1,z2):
        for x,s in zip([x1,x2,y1,y2,z1,z2],self.sliderBounds):
            flag = s.blockSignals(True)
            s.setValue(x*100)
            s.blockSignals(flag)


    def boundsChanged(self):
        bounds = [s.value()/100. for s in self.sliderBounds]
        self._boundsChanged.emit(*bounds)


    def alphaPowChanged(self):
        alphaPow = 100.*(self.sliderAlphaPow.value()/100.)**3
        self._alphaPowChanged.emit(alphaPow)

    def stackUnitsChanged(self):
        try:
            stackUnits = [float(e.text()) for e in self.stackEdits]
            self._stackUnitsChanged.emit(*stackUnits)
        except Exception as e:
            print "couldnt parse text"
            print e
        
class MainWindow(QtGui.QMainWindow):

    def __init__(self, ):
        super(QtGui.QMainWindow,self).__init__()

        self.resize(500, 300)
        self.setWindowTitle('Test')

        self.settings = VolumeSettingsPanel()
        self.setCentralWidget(self.settings)
        self.setStyleSheet("background-color:black;")

    def close(self):
        QtGui.qApp.quit()


if __name__ == '__main__':

    app = QtGui.QApplication(sys.argv)

    win = MainWindow()
    win.show()
    win.raise_()

    sys.exit(app.exec_())



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

setup(name='spimagine',
    version='0.1.3',
    description='renders spim data in 3D/4D',
    url='http://mweigert@bitbucket.org/mweigert/spimagine',
    author='Martin Weigert',
    author_email='mweigert@mpi-cbg.de',
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'numpy', 'scipy','Pillow',"PyOpenGL","pyopencl"
        ,"sortedcontainers"
    ],

    package_data={"spimagine":['volumerender/kernels/*',
                               'gui/shaders/*',
                               'gui/images/*',
                               'colormaps/*',
                               'data/*',
                               'lib/*']},
      entry_points = {
          'console_scripts': [
              'spimagine_render = spimagine.bin.spim_render:main'
          ],
    'gui_scripts': [
        'spimagine = spimagine.bin.spimagine_gui:main'
    ]
        }
)



"""
defines mesh object that are to be rendered with OpenGL

mweigert@mpi-cbg.de

"""

import numpy as np


class Mesh(object):
    """
    A mesh object is defined by its vertices, normals and edge/facecolors
    """
    def __init__(self,
                 vertices = [[0,0,0],[1,0,0],[0,1,0]],
                 normals = None,
                 facecolor = (1.,1.,1.),
                 edgecolor = None,
                 alpha = 1.,
                 light = [1,1,-1]):
        """
        vertices are a list of 3d coordinates like [[0,0,0], [1,0,0],...]
        where 3 consecutive coordinates define a triangle to be rendered.

        normals, facecolor , edgecolor,  alpha are optional

        face/edgecolor are (r,g,b) colors or None if they should not be drawn at all

        if normals and light are given (and facecolor is not None), a simple
        Phong shading is applied
        """

        assert len(vertices)%3==0

        self.vertices = vertices
        self.edges = []
        self.alpha = np.clip(alpha,0,1.)

        #FIXME: this includes every edge twice which is stupid
        for i in range(len(vertices)/3):
            v1,v2, v3 = vertices[3*i], vertices[3*i+1], vertices[3*i+2]
            self.edges.append(v1)
            self.edges.append(v2)
            self.edges.append(v1)
            self.edges.append(v3)
            self.edges.append(v2)
            self.edges.append(v3)


        self.normals = normals

        self.facecolor = facecolor
        self.edgecolor = edgecolor


        self.light = light


class EllipsoidMesh(Mesh):
    def __init__(self, rs = (1.,.5,.5),
                 pos = (0,0,0),
                 n_phi = 30,
                 n_theta = 20,
                 facecolor = (1.,1.,1.),
                 edgecolor = None,
                 alpha = 1.,
                 light = [1,1,-1]):
        """creates an ellipsoidal mesh at pos with half axes rs = (rx,ry,rz)
        of equally distributing points n_phi x n_theta"""

        vertices, normals = EllipsoidMesh.create_verts(rs, pos, n_phi, n_theta)

        super(EllipsoidMesh,self).__init__(vertices = vertices,
                                           normals = normals,
                                           facecolor = facecolor,
                                           edgecolor = edgecolor,
                                           alpha = alpha,
                                           light = light)
    @staticmethod
    def create_verts(rs, pos, n_phi, n_theta):
        ts = np.linspace(0,np.pi,n_theta)
        ps = np.linspace(0,2.*np.pi,n_phi+1)

        T,P = np.meshgrid(ts,ps, indexing = "ij")
        rx,ry,rz = rs
        xs = np.array([rx*np.cos(P)*np.sin(T),ry*np.sin(P)*np.sin(T),rz*np.cos(T)])

        verts = []
        normals = []
        for i in range(len(ts)-1):
            for j in range(len(ps)-1):
                verts.append(xs[:,i,j])
                verts.append(xs[:,i+1,j])
                verts.append(xs[:,i+1,j+1])
                verts.append(xs[:,i,j])
                verts.append(xs[:,i+1,j+1])
                verts.append(xs[:,i,j+1])

                #FIXME, wrong for rx != ry ....
                normals.append(1.*xs[:,i,j]/rx)
                normals.append(1.*xs[:,i+1,j]/rx)
                normals.append(1.*xs[:,i+1,j+1]/rx)
                normals.append(1.*xs[:,i,j]/rx)
                normals.append(1.*xs[:,i,j+1]/rx)
                normals.append(1.*xs[:,i+1,j+1]/rx)

        return  np.array(pos)+np.array(verts), np.array(normals)


class SphericalMesh(EllipsoidMesh):
    def __init__(self,r = 1.,
                 pos = (0,0,0),
                 n_phi = 30,
                 n_theta = 20,
                 facecolor = (1.,1.,1.),
                 edgecolor = None,
                 alpha = 1.,
                 light = [1,1,-1]):
        """creates a spherical mesh at pos with radius r
        of equally distributing points n_phi x n_theta"""


        super(SphericalMesh,self).__init__(rs = (r,)*3,
                                           pos = pos,
                                           n_phi = n_phi, n_theta = n_theta,
                                           facecolor = facecolor,
                                           edgecolor = edgecolor,
                                           alpha = alpha,
                                           light = light)


if __name__ == '__main__':
    print Mesh().vertices
    print EllipsoidMesh().vertices
    print SphericalMesh().facecolor



"""

receive input data of Loics Egg3d controller and emit qt signals

"""

from PyQt4 import QtCore,QtGui

import socket

import select
import numpy as np


import time



def empty_socket(sock):
    """remove the data present on the socket"""
    input = [sock]
    while 1:
        inputready, o, e = select.select(input,[],[], 0.0)
        if len(inputready)==0: break
        for s in inputready: s.recv(4096)


def getEggData(s):


    while s.recv(1) != "[":
        pass
    tmp = "["

    while tmp.find(']') == -1:
        tmp += s.recv(1)

    empty_socket(s)

    try:
        val = eval("np.array(%s)"%tmp)
    except:
        val = np.array([1,0,0,0,0,0,0,0,0,0])

    return val




class Egg3dListener(QtCore.QThread):
    _quaternionChanged =  QtCore.pyqtSignal(float,float,float,float)
    _zoomChanged = QtCore.pyqtSignal(int)

    def __init__(self):
        super(Egg3dListener,self).__init__()

    def set_socket(self,s):
        self.socket = s

    def run(self):
        print "Egg3dlistener started"
        self.isActive = True
        self.vals = []
        while self.isActive:
            try:
                val = getEggData(self.socket)

                #acceleration
                if sum(abs(val[4:7]))>.5:
                    # button pressed?
                    self._zoomChanged.emit(2.*(val[-1]>125)-1)

                #rotation
                quat = val[:4]
                self._quaternionChanged.emit(*val[:4])
                    
            except  Exception as e:
                print "could not read", e
            time.sleep(0.005)




class Egg3dController(QtCore.QObject):

    def __init__(self,):
        super(Egg3dController,self).__init__()
        self.listener = Egg3dListener()

    def _reset(self, port=4444):
        try:
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.listener.set_socket(self.socket)
            self.socket.connect(("localhost",port))
            # self.listener._zoomChanged.connect(self.foo)
            print "connection with Egg3d established!"
        except Exception as e:
            print "Couldnt connect with port %i:  %s"%(port,e)
            raise Exception("Connection refused at port %s"%port)


    def start(self):
        print "starting Egg3dController..."

        self._reset()
        self.listener.start()

    def stop(self):
        print "stopping Egg3dController..."
        self.listener.isActive = False
        # self.socket.shutdown(socket.SHUT_RDWR)
        self.socket.close()


    def foo(self,zoom):
        print zoom




if __name__ == '__main__':


    egg = Egg3dController()




    egg.start()

    time.sleep(20)


    egg.stop()

    # time.sleep(1)

    # egg.start()



    # s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    # s.connect(("localhost",4444))


    # while True:
    #     d = getEggData(s)
    #     print d



#!/usr/bin/env python

"""

The rendering widget

It renderes a projection via the OpenCL (defined in volume_render.py)
into a texture which is drawn by simple OpenGL calls onto the canvas.

It should handle all user interaction via a transformation model.


author: Martin Weigert
email: mweigert@mpi-cbg.de
"""




"""
understanding glBlendFunc:

first color:     d
second color:    s
resulting color: c

c = s*S + d*D

where S and D are set with glBlendFunc(S,D)

e.g. glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)

c = s*s.w + d*(1-s.w)



"""

import logging

logger = logging.getLogger(__name__)




import sys
import os
from PyQt4 import QtCore
from PyQt4 import QtGui
from PyQt4 import QtOpenGL

from OpenGL.GL import *
from OpenGL.GL import shaders

import spimagine

from spimagine.volumerender.volume_render import VolumeRenderer
from spimagine.utils.transform_matrices import *
from spimagine.models.transform_model import TransformModel

from spimagine.models.data_model import DataModel


from spimagine.gui.mesh import Mesh, SphericalMesh, EllipsoidMesh


import numpy as np
from spimagine.gui.gui_utils import *


# on windows numpy.linalg.inv crashes without notice, so we have to import scipy.linalg
if os.name == "nt":
    from scipy import linalg
else:
    from numpy import linalg

import time
from spimagine.utils.quaternion import Quaternion

# from spimagine.shaders import vertShaderTex, fragShaderTex, vertShaderCube, fragShaderCube

# logger.setLevel(logging.DEBUG)

def _next_golden(n):
    res = round((np.sqrt(5)-1.)/2.*n)
    return int(round((np.sqrt(5)-1.)/2.*n))


def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:
        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)




class GLWidget(QtOpenGL.QGLWidget):
    _dataModelChanged = QtCore.pyqtSignal()
    _foo= QtCore.pyqtSignal()

    _BACKGROUND_BLACK = (0.,0.,0.,0.)
    _BACKGROUND_WHITE = (1.,1.,1.,0.)


    def __init__(self, parent=None, N_PREFETCH = 0,**kwargs):
        logger.debug("init")

        super(GLWidget,self).__init__(parent,**kwargs)

        self.parent= parent
        self.texture_LUT = None

        self.setAcceptDrops(True)

        self.renderer = VolumeRenderer((spimagine.config.__DEFAULTWIDTH__,
                                        spimagine.config.__DEFAULTWIDTH__))

        self.renderer.set_projection(mat4_perspective(60,1.,.1,100))
        # self.renderer.set_projection(projMatOrtho(-2,2,-2,2,-10,10))

        self.output = np.zeros([self.renderer.height,self.renderer.width],dtype = np.float32)
        self.output_alpha = np.zeros([self.renderer.height,self.renderer.width],dtype = np.float32)

        self.sliceOutput = np.zeros((100,100),dtype = np.float32)

        self.setTransform(TransformModel())

        self.renderTimer = QtCore.QTimer(self)
        self.renderTimer.setInterval(10)
        self.renderTimer.timeout.connect(self.onRenderTimer)
        self.renderTimer.start()
        self.renderedSteps = 0

        self.N_PREFETCH = N_PREFETCH

        self.NSubrenderSteps = 1

        self.dataModel = None

        self.meshes = []

        # self.setMouseTracking(True)

        self._dataModelChanged.connect(self.dataModelChanged)

        self.refresh()


    def set_background_mode_black(self, mode_back = True):
        self._background_mode_black = mode_back
        self.refresh()




    def setModel(self,dataModel):
        logger.debug("setModel to %s"%dataModel)
        if self.dataModel is None or (self.dataModel != dataModel):
                self.dataModel = dataModel
                self.transform.setModel(dataModel)
                self.dataModel._dataSourceChanged.connect(self.dataSourceChanged)
                self.dataModel._dataPosChanged.connect(self.dataPosChanged)
                self._dataModelChanged.emit()



    def foo(self):
        print "fooooooooooooooo"



    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.accept()
        else:
            event.ignore()

    def dropEvent(self, event):

        for url in event.mimeData().urls():

            path = url.toLocalFile().toLocal8Bit().data()

            if spimagine.config.__SYSTEM_DARWIN__:

                path = spimagine.config._parseFileNameFix(path)

            self.setCursor(QtCore.Qt.BusyCursor)

            if self.dataModel:
                self.dataModel.loadFromPath(path, prefetchSize = self.N_PREFETCH)
            else:
                self.setModel(DataModel.fromPath(path, prefetchSize = self.N_PREFETCH))

            self.setCursor(QtCore.Qt.ArrowCursor)


    def set_colormap(self,name):
        """name should be either jet, hot, gray coolwarm"""

        try:
            arr = spimagine.config.__COLORMAPDICT__[name]
            self._set_colormap_array(arr)
        except KeyError:
            print "could not load colormap '%s'"%name
            print "valid names: %s"%spimagine.config.__COLORMAPDICT__.keys()


    def set_colormap_rgb(self,color=[1.,1.,1.]):
        self._set_colormap_array(np.outer(np.linspace(0,1.,255),np.array(color)))


    def _set_colormap_array(self,arr):
        """arr should be of shape (N,3) and gives the rgb components of the colormap"""

    
        self.makeCurrent()
        self.texture_LUT = fillTexture2d(arr.reshape((1,)+arr.shape),self.texture_LUT)
        self.refresh()

    def _shader_from_file(self,fname_vert,fname_frag):
        shader = QtOpenGL.QGLShaderProgram()
        shader.addShaderFromSourceFile(QtOpenGL.QGLShader.Vertex,fname_vert)
        shader.addShaderFromSourceFile(QtOpenGL.QGLShader.Fragment,fname_frag)
        shader.link()
        shader.bind()
        logger.debug("GLSL program log:%s",shader.log())
        return shader

    def initializeGL(self):

        self.resized = True

        logger.debug("initializeGL")

        self.programTex = self._shader_from_file(absPath("shaders/texture.vert"),
                                                 absPath("shaders/texture.frag"))

        self.programCube = self._shader_from_file(absPath("shaders/box.vert"),
                                                  absPath("shaders/box.frag"))

        self.programSlice = self._shader_from_file(absPath("shaders/slice.vert"),
                                                   absPath("shaders/slice.frag"))

        self.programMesh = self._shader_from_file(absPath("shaders/mesh.vert"),
                                                  absPath("shaders/mesh.frag"))

        self.programMeshLight = self._shader_from_file(
            absPath("shaders/mesh_light.vert"),
            absPath("shaders/mesh_light.frag"))



        self.texture = None
        self.textureAlpha = None
        self.textureSlice = None

        self.quadCoord = np.array([[-1.,-1.,0.],
                           [1.,-1.,0.],
                           [1.,1.,0.],
                           [1.,1.,0.],
                           [-1.,1.,0.],
                           [-1.,-1.,0.]])

        self.quadCoordTex = np.array([[0,0],
                           [1.,0.],
                           [1.,1.],
                           [1.,1.],
                           [0,1.],
                           [0,0]])

        # self.cubeCoords = create_cube_coords([-1,1,-1,1,-1,1])

        self.set_colormap(spimagine.config.__DEFAULTCOLORMAP__)

        glEnable( GL_BLEND )

        # glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)


        # glLineWidth(1.0);
        glBlendFunc(GL_ONE,GL_ONE)

        glEnable( GL_LINE_SMOOTH );
        glDisable(GL_DEPTH_TEST)

        #self.set_background_color(0,0,0,.0)
        self.set_background_mode_black(True)
        # self.set_background_color(1,1,1,.6)




    def setTransform(self, transform):
        self.transform = transform
        self.transform._transformChanged.connect(self.refresh)
        self.transform._stackUnitsChanged.connect(self.setStackUnits)
        self.transform._boundsChanged.connect(self.setBounds)


    def dataModelChanged(self):
        logger.debug("+++++++++ data model changed")
        logger.debug("dataModelchanged: min %s max %s"%(np.amin(self.dataModel[0]),
                                                         np.amax(self.dataModel[0])))
        if self.dataModel:
            self.renderer.set_data(self.dataModel[0], autoConvert = True)
            self.transform.reset(minVal = np.amin(self.dataModel[0]),
                                 maxVal = np.amax(self.dataModel[0]),
                                 stackUnits= self.dataModel.stackUnits())
            self.meshes = []
            self.refresh()


    def set_background_color(self,r,g,b,a=1.):
        self._background_color = (r,g,b,a)
        glClearColor(r,g,b,a)
        


    def dataSourceChanged(self):
        # print "SETDATA: ", self.dataModel[0].shape
        logger.debug("dataSourcechanged: min %s max %s"%(np.amin(self.dataModel[0]),
                                                         np.amax(self.dataModel[0])))
        self.renderer.set_data(self.dataModel[0],autoConvert = True)
        self.transform.reset(minVal = np.amin(self.dataModel[0]),
                             maxVal = np.amax(self.dataModel[0]),
                             stackUnits= self.dataModel.stackUnits())

        self.refresh()


    def setBounds(self,x1,x2,y1,y2,z1,z2):
        self.cubeCoords = create_cube_coords([x1,x2,y1,y2,z1,z2])
        self.renderer.set_box_boundaries([x1,x2,y1,y2,z1,z2])

    def setStackUnits(self,px,py,pz):
        logger.debug("setStackUnits to %s"%[px,py,pz])
        self.renderer.set_units([px,py,pz])


    def dataPosChanged(self,pos):
        self.renderer.update_data(self.dataModel[pos])
        self.refresh()


    def refresh(self):
        # if self.parentWidget() and self.dataModel:
        #     self.parentWidget().setWindowTitle("SpImagine %s"%self.dataModel.name())
        
        self.renderUpdate = True
        self.renderedSteps = 0

    def resizeGL(self, width, height):

        height = max(10,height)

        self.width , self.height = width, height

        #make the viewport squarelike
        w = max(width,height)
        glViewport((width-w)/2,(height-w)/2,w,w)

        self.resized = True

    def add_mesh(self, mesh = SphericalMesh()):
        """
        adds a mesh with vertices and facecolor/edgecolor to be drawn

        mesh is an instance of spimagine.gui.Mesh, e.g.

        mesh = Mesh(vertices = [[0,1,0],[0,1,0],...],
                    normals = [[0,1,0],[0,1,0],...],
                    facecolor = (1.,.4,.4,.2),
                    edgecolor = None,...)

        there are some predefined meshes like
        SphericalMesh, EllipsoidMesh ...
        """


        self.meshes.append(mesh)

        #sort according to opacity as the opaque objects should be drawn first
        self.meshes.sort(key = lambda x:x.alpha, reverse = True)


    # def add_surface_sphere(self,pos = (0,0,0),
    #                        radius = .2,
    #                        Nphi = 40, Ntheta = 30,
    #                        facecolor = (1.,1.,1.,1.),
    #                        edgecolor = None):
    #
    #     coords = np.array(pos)+create_sphere_coords(radius,radius,radius,Nphi, Ntheta)
    #     self.add_surface(coords,
    #                      facecolor = facecolor,
    #                      edgecolor = edgecolor)
    #
    # def add_surface_ellipsoid(self,pos = (0,0,0),
    #                        rs = (.2,.2,.2),
    #                        Nphi = 40, Ntheta = 30,
    #                           facecolor = (1.,1.,1.,1.),
    #                        edgecolor = None):
    #     coords = np.array(pos)+create_sphere_coords(rs[0],rs[1],rs[2],Nphi, Ntheta)
    #     self.add_surface(coords,
    #                      edgecolor = edgecolor,
    #                      facecolor = facecolor)
    #

    def _paintGL_render(self):
        # Draw the render texture
        self.programTex.bind()

        self.texture = fillTexture2d(self.output,self.texture)

        glEnable(GL_TEXTURE_2D)
        glDisable(GL_DEPTH_TEST)

        self.programTex.enableAttributeArray("position")
        self.programTex.enableAttributeArray("texcoord")
        self.programTex.setAttributeArray("position", self.quadCoord)
        self.programTex.setAttributeArray("texcoord", self.quadCoordTex)


        self.programTex.setUniformValue("is_mode_black",self._background_mode_black)
        glActiveTexture(GL_TEXTURE0)
        glBindTexture(GL_TEXTURE_2D, self.texture)
        self.programTex.setUniformValue("texture",0)

        glActiveTexture(GL_TEXTURE1)
        glBindTexture(GL_TEXTURE_2D, self.textureAlpha)
        self.programTex.setUniformValue("texture_alpha",1)

        glActiveTexture(GL_TEXTURE2)
        glBindTexture(GL_TEXTURE_2D, self.texture_LUT)
        self.programTex.setUniformValue("texture_LUT",2)

        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

        glDrawArrays(GL_TRIANGLES,0,len(self.quadCoord))




    def _paintGL_slice(self):
        #draw the slice
        self.programSlice.bind()
        self.programSlice.setUniformValue("mvpMatrix",QtGui.QMatrix4x4(*self.finalMat.flatten()))
        self.programSlice.enableAttributeArray("position")

        pos, dim = self.transform.slicePos,self.transform.sliceDim

        coords = slice_coords(1.*pos/self.dataModel.size()[2-dim+1],dim)

        texcoords = [[0.,0.],[1,0.],[1.,1.],
                     [1.,1.],[0.,1.],[0.,0.]]



        self.programSlice.setAttributeArray("position", coords)
        self.programSlice.setAttributeArray("texcoord", texcoords)

        self.textureSlice = fillTexture2d(self.sliceOutput,self.textureSlice)


        glActiveTexture(GL_TEXTURE0)
        glBindTexture(GL_TEXTURE_2D, self.textureSlice)
        self.programSlice.setUniformValue("texture",0)


        glActiveTexture(GL_TEXTURE1)
        glBindTexture(GL_TEXTURE_2D, self.texture_LUT)
        self.programSlice.setUniformValue("texture_LUT",1)


        glDrawArrays(GL_TRIANGLES,0,len(coords))





    def _paintGL_box(self):

        # Draw the cube
        self.programCube.bind()
        self.programCube.setUniformValue("mvpMatrix",QtGui.QMatrix4x4(*self.finalMat.flatten()))
        self.programCube.enableAttributeArray("position")

        if self._background_mode_black:
            self.programCube.setUniformValue("color",
                                                 QtGui.QVector4D(1,1,1,0.6))
        else:
            self.programCube.setUniformValue("color",
                                                 QtGui.QVector4D(0,0,0,0.6))

        self.programCube.setAttributeArray("position", self.cubeCoords)


        glActiveTexture(GL_TEXTURE0)
        glBindTexture(GL_TEXTURE_2D, self.textureAlpha)
        self.programCube.setUniformValue("texture_alpha",0)

        glEnable(GL_DEPTH_TEST)
        # glBlendFunc(GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA)
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
        glDrawArrays(GL_LINES,0,len(self.cubeCoords))
        glDisable(GL_DEPTH_TEST)


    def _paintGL_mesh(self, mesh):
        """
        paint a mesh (which has all the coordinates and colors in it
        """
        # Draw the cube


        #whether we use phong lightning or not
        prog = self.programMesh


        prog.bind()
        prog.setUniformValue("mvpMatrix",
                        QtGui.QMatrix4x4(*self.finalMat.flatten()))

        glDisable(GL_DEPTH_TEST)
        glEnable( GL_BLEND )
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)

        if mesh.facecolor:

            r,g,b = mesh.facecolor
            a = mesh.alpha

            prog.enableAttributeArray("position")
            prog.setAttributeArray("position", mesh.vertices)

            prog.setUniformValue("color",
                                         QtGui.QVector4D(r,g,b,a))


            glDrawArrays(GL_TRIANGLES,0,len(mesh.vertices))

            glDisable(GL_DEPTH_TEST)

        if mesh.edgecolor:
            r,g,b = mesh.edgecolor
            a = mesh.alpha

            prog.enableAttributeArray("position")
            prog.setAttributeArray("position", mesh.edges)

            prog.setUniformValue("color",
                                         QtGui.QVector4D(r,g,b,a))


            glDrawArrays(GL_LINES,0,len(mesh.edges))


    #FIXME this should be the new default....
    def _paintGL_mesh_light(self, mesh):
        """
        paint a mesh (which has all the coordinates and colors in it
        """
        # Draw the cube


        #whether we use phong lightning or not
        if mesh.light:
            prog = self.programMeshLight
        else:
            prog = self.programMesh


        prog.bind()
        prog.setUniformValue("mvpMatrix",
                        QtGui.QMatrix4x4(*self.finalMat.flatten()))

        glDisable(GL_DEPTH_TEST)
        glEnable( GL_BLEND )
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)

        if mesh.facecolor:
            r,g,b = mesh.facecolor
            a = mesh.alpha

            prog.enableAttributeArray("position")
            prog.setAttributeArray("position", mesh.vertices)

            prog.setUniformValue("color",
                                         QtGui.QVector4D(r,g,b,a))

            if mesh.light:
                prog.enableAttributeArray("normal")
                prog.setAttributeArray("normal", mesh.normals)
                prog.setUniformValue("light",
                                         QtGui.QVector3D(*mesh.light))


            # glDisable(GL_DEPTH_TEST)
            # glEnable( GL_BLEND )
            # glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
            # #glBlendFunc(GL_SRC_ALPHA,GL_ONE)
            # print mesh.alpha
            # # if mesh.alpha>=1.:
            #     glEnable(GL_DEPTH_TEST)
            #     glDisable( GL_BLEND )
            #     glDepthFunc(GL_LESS)
            #     glBlendFunc(GL_SRC_ALPHA,GL_ONE)
            #
            # else:
            #     glEnable(GL_DEPTH_TEST)
            #     glEnable( GL_BLEND )
            #     glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)



            glDrawArrays(GL_TRIANGLES,0,len(mesh.vertices))

            glDisable(GL_DEPTH_TEST)

        if mesh.edgecolor:
            r,g,b = mesh.edgecolor
            a = mesh.alpha

            prog.enableAttributeArray("position")
            prog.setAttributeArray("position", mesh.edges)

            prog.setUniformValue("color",
                                         QtGui.QVector4D(r,g,b,a))

            # if mesh.alpha>=1.:
            #     glEnable(GL_DEPTH_TEST)
            # else:
            #     glDisable(GL_DEPTH_TEST)

            # glEnable(GL_DEPTH_TEST)
            # glEnable( GL_BLEND )
            # glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
            # print mesh.alpha

            glDrawArrays(GL_LINES,0,len(mesh.edges))



    def paintGL(self):

        self.makeCurrent()

        if not glCheckFramebufferStatus(GL_FRAMEBUFFER)==GL_FRAMEBUFFER_COMPLETE:
            return

        #hack
        if self.resized:
            w = max(self.width,self.height)
            glViewport((self.width-w)/2,(self.height-w)/2,w,w)
            self.resized = False

        if self._background_mode_black:
            glClearColor(*self._BACKGROUND_BLACK)
        else:
            glClearColor(*self._BACKGROUND_WHITE)

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

        modelView = self.transform.getModelView()

        proj = self.transform.getProjection()



        self.finalMat = np.dot(proj,modelView)

        if self.dataModel:


            self.textureAlpha = fillTexture2d(self.output_alpha,self.textureAlpha)

            if self.transform.isBox:
                self._paintGL_box()

            if self.transform.isSlice and self.sliceOutput is not None:
                self._paintGL_slice()

            self._paintGL_render()

        for m in self.meshes:
            self._paintGL_mesh(m)




    def render(self):
        logger.debug("render")



        if self.dataModel:
            
            self.renderer.set_modelView(self.transform.getUnscaledModelView())
            self.renderer.set_projection(self.transform.getProjection())
            self.renderer.set_min_val(self.transform.minVal)

            self.renderer.set_max_val(self.transform.maxVal)
            self.renderer.set_gamma(self.transform.gamma)
            self.renderer.set_alpha_pow(self.transform.alphaPow)

            self.renderer.set_occ_strength(self.transform.occ_strength)
            self.renderer.set_occ_radius(self.transform.occ_radius)
            self.renderer.set_occ_n_points(self.transform.occ_n_points)

            if self.transform.isIso:
                renderMethod = "iso_surface"
                
            else:
                renderMethod = "max_project"

            self.renderer.render(method = renderMethod, return_alpha = True, numParts = self.NSubrenderSteps, currentPart = (self.renderedSteps*_next_golden(self.NSubrenderSteps)) %self.NSubrenderSteps)
            self.output, self.output_alpha = self.renderer.output, self.renderer.output_alpha

            if self.transform.isSlice:
                if self.transform.sliceDim==0:
                    out = self.dataModel[self.transform.dataPos][:,:,self.transform.slicePos]
                elif self.transform.sliceDim==1:
                    out = self.dataModel[self.transform.dataPos][:,self.transform.slicePos,:]
                elif self.transform.sliceDim==2:
                    out = self.dataModel[self.transform.dataPos][self.transform.slicePos,:,:]

                min_out, max_out = np.amax(out),np.amin(out)
                if max_out>min_out:
                    self.sliceOutput = (1.*(out-min_out)/(max_out-min_out))
                else:
                    self.sliceOutput = np.zeros_like(out)


    def getFrame(self):
        self.render()
        self.paintGL()
        glFlush()
        im = self.grabFrameBuffer()
        im = im.convertToFormat(QtGui.QImage.Format_RGB32)


        width = im.width()
        height = im.height()

        ptr = im.bits()
        ptr.setsize(im.byteCount())
        arr = np.array(ptr).reshape(height, width, 4)  #  Copies the data
        return arr[...,[2,1,0,3]].copy()


    def saveFrame(self,fName):
        """FIXME: scaling behaviour still hast to be implemented (e.g. after setGamma)"""
        logger.info("saving frame as %s", fName)

        #has to be png

        name, ext = os.path.splitext(fName)
        if ext != ".png":
            fName = name+".png"

        self.render()
        self.paintGL()
        glFlush()
        im = self.grabFrameBuffer()
        im.save(fName)

        
    def onRenderTimer(self):
        # if self.renderUpdate:
        #     self.render()
        #     self.renderUpdate = False
        #     self.updateGL()
        if self.renderedSteps<self.NSubrenderSteps:
            # print ((self.renderedSteps*7)%self.NSubrenderSteps)
            s = time.time()
            self.render()
            logger.debug("time to render:  %.2f"%(1000.*(time.time()-s)))
            self.renderedSteps +=1 
            self.updateGL()



    def wheelEvent(self, event):
        """ self.transform.zoom should be within [1,2]"""
        newZoom = self.transform.zoom * 1.2**(event.delta()/1400.)
        newZoom = np.clip(newZoom,.4,3)
        self.transform.setZoom(newZoom)

        logger.debug("newZoom: %s",newZoom)
        # self.refresh()


    def posToVec3(self,x,y, r0 = .8, isRot = True ):
        x, y = 2.*x/self.width-1.,1.-2.*y/self.width
        r = np.sqrt(x*x+y*y)
        if r>r0-1.e-7:
            x,y = 1.*x*r0/r, 1.*y*r0/r
        z = np.sqrt(max(0,r0**2-x*x-y*y))
        if isRot:
            M = np.linalg.inv(self.transform.quatRot.toRotation3())
            x,y,z = np.dot(M,[x,y,z])

        return x,y,z

    def posToVec2(self,x,y):
        x, y = 2.*x/self.width-1.,1.-2.*y/self.width
        return x,y


    def mousePressEvent(self, event):
        super(GLWidget, self).mousePressEvent(event)

        if event.buttons() == QtCore.Qt.LeftButton:
            self._x0, self._y0, self._z0 = self.posToVec3(event.x(),event.y())

        if event.buttons() == QtCore.Qt.RightButton:
            (self._x0, self._y0), self._invRotM = self.posToVec2(event.x(),event.y()), linalg.inv(self.transform.quatRot.toRotation3())

        # self.setCursor(QtCore.Qt.ClosedHandCursor)

    def mouseReleaseEvent(self, event):
        super(GLWidget, self).mouseReleaseEvent(event)

        # self.setCursor(QtCore.Qt.ArrowCursor)

    def mouseMoveEvent(self, event):

        # c = append(self.cubeCoords,ones(24)[:,newaxis],axis=1)
        # cUser = dot(c,self.finalMat)
        # cUser = cUser[:,:3]/cUser[:,-1,newaxis]
        # print self.finalMat
        # print c[0], cUser[0]
        # Rotation
        if event.buttons() == QtCore.Qt.LeftButton:

            x1,y1,z1 = self.posToVec3(event.x(),event.y())
            n = np.cross(np.array([self._x0,self._y0,self._z0]),np.array([x1,y1,z1]))
            nnorm = linalg.norm(n)
            if np.abs(nnorm)>=1.:
                nnorm *= 1./np.abs(nnorm)
            w = np.arcsin(nnorm)
            n *= 1./(nnorm+1.e-10)
            q = Quaternion(np.cos(.5*w),*(np.sin(.5*w)*n))
            self.transform.setQuaternion(self.transform.quatRot*q)

        #Translation
        if event.buttons() == QtCore.Qt.RightButton:
            x, y = self.posToVec2(event.x(),event.y())

            dx, dy, foo = np.dot(self._invRotM,[x-self._x0, y-self._y0,0])

            self.transform.addTranslate(dx,dy,foo)
            self._x0,self._y0 = x,y

        self.refresh()



def test_sphere():
    from data_model import DataModel, NumpyData, SpimData, TiffData

    app = QtGui.QApplication(sys.argv)

    win = GLWidget(size=QtCore.QSize(500,500))


    x = np.linspace(-1,1,128)
    Z,Y,X = np.meshgrid(x,x,x)
    # R = sqrt(Z**2+Y**2+(X-.35)**2)
    # R2 = sqrt(Z**2+Y**2+(X+.35)**2)

    # d = 100.*exp(-10*R**2)+.0*np.random.normal(0,1.,X.shape)

    # d += 100.*exp(-10*R2**2)+.0*np.random.normal(0,1.,X.shape)

    Ns = 5
    r = .6
    phi = np.linspace(0,2*pi,Ns+1)[:-1]
    d = np.zeros_like(X)
    for p in phi:
        d += 100.*np.exp(-10*(Z**2+(Y-r*np.sin(p))**2+(X-r*np.cos(p))**2))


    win.setModel(DataModel(NumpyData(d)))

    win.transform.setValueScale(0,40)


    win.show()

    win.raise_()

    sys.exit(app.exec_())

def test_demo():

    from data_model import DataModel, DemoData, SpimData, TiffData, NumpyData
    
    app = QtGui.QApplication(sys.argv)

    win = GLWidget(size=QtCore.QSize(800,800))

    win.setModel(DataModel(DemoData()))

    win.show()


    win.raise_()

    sys.exit(app.exec_())


def test_demo_simple():

    from spimagine import DataModel, DemoData
    
    app = QtGui.QApplication(sys.argv)

    win = GLWidget(size=QtCore.QSize(800,800))

    win.setModel(DataModel(DemoData()))
    
    win.show()
    
    win.raise_()

    sys.exit(app.exec_())


def test_surface():
    from spimagine import DataModel, DemoData

    app = QtGui.QApplication(sys.argv)

    win = GLWidget(size=QtCore.QSize(800,800))


    win.setModel(DataModel(DemoData()))

    # win.add_surface_sphere((0,0,0), 1., facecolor = (.0,.3,1.,.5),
    #                                 Nphi = 30, Ntheta=20)

    # win.add_mesh(SphericalMesh(r = .8,
    #                            facecolor = (1.,0.,0.),
    #                            #edgecolor = (1.,1.,1.),
    #                            edgecolor = None,
    #                            alpha = .3))

    # win.add_mesh(EllipsoidMesh(rs = (.3,.6,.6),
    #                             pos = (0,0,-.5),
    #
    #                            facecolor = (0.,1.,1.),
    #                            #edgecolor = (1.,1.,1.),
    #                            edgecolor = None,
    #                            alpha = .4))
    #

    win.show()

    win.raise_()

    sys.exit(app.exec_())


if __name__ == '__main__':

    # test_sphere()

    #test_demo_simple()
    test_surface()



from jack_input import JackSource
from PyQt4 import QtCore
import numpy as np

class JackPlugin(QtCore.QObject):
    NHist = 100
    def __init__(self, transform, n_bank = 0):
        super(JackPlugin,self).__init__()
        self.n_bank = n_bank
        self.transform = transform
        self.s = JackSource()
        self.history = [1.]*self.NHist

    def start(self):
        self.s.start()
        
        self.timer = QtCore.QTimer(self)
        self.timer.setInterval(10)
        self.timer.timeout.connect(self.onTimer)
        self.timer.start()

    def onTimer(self):
        val = self.s.data[self.n_bank]
        
        avg = val/np.mean(self.history)

        self.transform.setZoom(1+.03*avg)

        # p = 2.*np.pi*np.random.uniform(0,1.)
        # t = np.arccos(2.*np.random.uniform(0,1.)-1.)
        # x,y,z = np.cos(p)*np.sin(t),np.sin(p)*np.sin(t),np.cos(t)
        # self.transform.addRotation(.01*(1.e-5+avg),x,y,z)
        
        self.history.pop(0)
        self.history.append(val)
        
if __name__ == '__main__':


    p = JackPlugin(None)

    p.start()
    
    while(1):
        pass
